from django.contrib import admin

from .models import Profile


class ProfileAdmin(admin.ModelAdmin):
    def get_readonly_fields(self, request, obj=None):
        #if obj is None:
        return ['sso_id', 'sso_rev']


admin.site.register(Profile, ProfileAdmin)



import logging

from django.contrib.auth.backends import ModelBackend
from django.core.exceptions import ObjectDoesNotExist

from django.conf import settings

from django_sso_app.core.utils import get_profile_model

from .utils import (get_sso_id,
                    get_profile_from_sso,
                    create_local_profile_from_sso,
                    update_local_profile_from_sso,
                    create_local_profile_from_headers,
                    get_user_groups_from_header,
                    update_user_groups)


logger = logging.getLogger("app")
Profile = get_profile_model()


class DjangoSsoAppBackend(ModelBackend):
    """
    See django.contrib.auth.backends.RemoteUserBackend.

    The jwt has an expected payload such as the following

        {
          "iss": "nxhnkMPKcpRWaTKwNOvGcLcs5MHJINOg",
          "user_id": 1,
          "rev": 0,
          "exp": 1528376222,
          "fp": "b28493a6a29a5a38973a8a3e3abe7f34"
        }
    """

    def authenticate(self, request, consumer_username, encoded_jwt,
                     decoded_jwt):
        logger.info("Authenticating consumer {}".format(consumer_username))

        if consumer_username is None:
            return None

        if settings.DJANGO_SSO_REPLICATE_PROFILE:
            if encoded_jwt is None:
                return None

        sso_id = get_sso_id(consumer_username)

        try:
            user_profile = Profile.objects.get(sso_id=sso_id)
            user = user_profile
            if settings.DJANGO_SSO_APP_USES_EXTERNAL_USER_PROFILE:
                user = user_profile.user

            if settings.DJANGO_SSO_REPLICATE_PROFILE:
                # must replicate user profile
                logger.info("Replicating user profile for {}".format(user))
                rev_changed = user_profile.sso_rev != decoded_jwt["rev"]
                first_access = not user.is_active and not user_profile.is_unsubscribed
                if rev_changed or first_access:
                    if rev_changed:
                        logger.info("Rev changed from {} to {} for {}, updating ..."
                                    .format(user_profile.sso_rev, decoded_jwt["rev"],
                                            consumer_username))
                    if first_access:
                        logger.info("{}'s first access, updating ...".format(
                            consumer_username))

                    # update local profile from SSO
                    sso_profile = get_profile_from_sso(sso_id=sso_id,
                                                       encoded_jwt=encoded_jwt)
                    update_local_profile_from_sso(sso_profile=sso_profile,
                                                  profile=user_profile)

                    logger.info("{} updated with latest data from SSO"
                                .format(consumer_username))
                else:
                    logger.info("Nothing changed for {}".format(consumer_username))
            else:
                # just updates user groups
                logger.info("Got user from headers {}".format(user))

            if settings.DJANGO_SSO_MANAGE_USER_GROUPS:
                # update user groups
                logger.info("Updating user groups for {}".format(user))

                consumer_groups = request.META.get(
                    settings.DJANGO_SSO_CONSUMER_GROUPS_HEADER, None)
                user_groups = get_user_groups_from_header(consumer_groups)

                update_user_groups(user, user_groups)

        except ObjectDoesNotExist:
            if settings.DJANGO_SSO_REPLICATE_PROFILE:
                # create local profile from SSO
                sso_profile = get_profile_from_sso(sso_id=sso_id,
                                                   encoded_jwt=encoded_jwt)

                user_profile = create_local_profile_from_sso(sso_profile=sso_profile)
                user = user_profile
                if settings.DJANGO_SSO_APP_USES_EXTERNAL_USER_PROFILE:
                    user = user_profile.user
            else:
                # create local profile from headers
                user = create_local_profile_from_headers(request)

        return user



from typing import Any

from allauth.account.adapter import DefaultAccountAdapter
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from django.conf import settings
from django.http import HttpRequest


class AccountAdapter(DefaultAccountAdapter):
    def is_open_for_signup(self, request: HttpRequest):
        return getattr(settings, "ACCOUNT_ALLOW_REGISTRATION", True)


class SocialAccountAdapter(DefaultSocialAccountAdapter):
    def is_open_for_signup(self, request: HttpRequest, sociallogin: Any):
        return getattr(settings, "ACCOUNT_ALLOW_REGISTRATION", True)



from django.conf import settings
from django.conf.urls import url
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import include, path
# pai
from django.utils import timezone
from django.views import defaults as default_views
from django.views.decorators.http import last_modified
from django.views.generic import TemplateView
from django.views.i18n import JavaScriptCatalog

from ...views import SwaggerSchemaView, APIRoot, StatsView

if settings.DJANGO_SSO_BACKEND_ENABLED:
    from .allauth import allauth_urlpatterns
    from .rest_auth import rest_auth_urlpatterns
    from .sso import sso_app_urlpatterns
elif settings.DJANGO_SSO_APP_ENABLED:
    from .sso import sso_app_urlpatterns

last_modified_date = timezone.now()
js_info_dict = {}

urlpatterns = [
    path("", TemplateView.as_view(template_name="pages/home.html"), name="home"),
    path(
        "about/", TemplateView.as_view(template_name="pages/about.html"), name="about"
    ),
    path("profile/", TemplateView.as_view(template_name="pages/profile.html"), name="profile"),

    # Django Admin, use {% url 'admin:index' %}
    path(settings.ADMIN_URL, admin.site.urls),

    # pai
    url(r'^i18n/', include('django.conf.urls.i18n')),
    url(r'^jsi18n/$', last_modified(lambda req, **kw: last_modified_date)(JavaScriptCatalog.as_view()), js_info_dict,
        name='javascript-catalog'),
]
api_urlpatterns = []

if settings.DJANGO_SSO_BACKEND_ENABLED:
    urlpatterns += allauth_urlpatterns
    api_urlpatterns += rest_auth_urlpatterns
elif settings.DJANGO_ALLAUTH_ENABLED:
    urlpatterns += [
        path("accounts/", include("allauth.urls")),
        # ! add social
    ]

# Your stuff: custom urls includes go here
urlpatterns += [
    # path("users/", include("django_sso_app.backend.users.urls", namespace="users")),
]

urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
if settings.DJANGO_SSO_APP_SERVER is not None:
    urlpatterns += staticfiles_urlpatterns()

if settings.DEBUG:
    # This allows the error pages to be debugged during development, just visit
    # these url in browser to see how these error pages look like.
    urlpatterns += [
        path(
            "400/",
            default_views.bad_request,
            kwargs={"exception": Exception("Bad Request!")},
        ),
        path(
            "403/",
            default_views.permission_denied,
            kwargs={"exception": Exception("Permission Denied")},
        ),
        path(
            "404/",
            default_views.page_not_found,
            kwargs={"exception": Exception("Page not Found")},
        ),
        path("500/", default_views.server_error),
    ]

    if "debug_toolbar" in settings.INSTALLED_APPS:
        import debug_toolbar

        urlpatterns = [path("__debug__/", include(debug_toolbar.urls))] + urlpatterns

# pai
if settings.DJANGO_SSO_BACKEND_ENABLED or settings.DJANGO_SSO_APP_ENABLED:
    api_urlpatterns += sso_app_urlpatterns


api_urlpatterns += [
    url(r'^api/v1/_stats/$', StatsView.as_view(), name="stats"),

    # your api here
]

urlpatterns += api_urlpatterns

urlpatterns += [
    url(r'^api/v1/ui/$', APIRoot.as_view(), name="drf"),
    url(r'^api/v1/$', SwaggerSchemaView.as_view(patterns=api_urlpatterns), name="swagger")
]

"""
if settings.DJANGO_SSO_BACKEND_ENABLED:
    urlpatterns += [
        url(r'^api/v1/auth/', AuthAPIRoot.as_view(), name='authapi'),
    ]
"""



import logging
import uuid

from django.contrib.auth.hashers import identify_hasher, get_hasher
from django.contrib.auth.models import AbstractUser
from django.db import models
from django.db import transaction
from django.db.models import F
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from django_sso_app.core.common import DJANGO_SSO_REQUIRED_USER_FIELDS

logger = logging.getLogger('users')


class AbstractUserModel(AbstractUser):
    """
    Abstract user model, the fields below should be defined by 'django.contrib.auth.models.AbstractUser'

    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    """

    class Meta:
        abstract = True

    # set unique=False on first migration to enable model swap
    sso_id = models.UUIDField(_('sso id'), unique=True, db_index=True, default=uuid.uuid4, editable=False)
    sso_rev = models.PositiveIntegerField(_('sso revision'), default=1)

    unsubscribed_at = models.DateTimeField(_('unsubscription date'), null=True, blank=True)
    apigw_consumer_id = models.CharField(_('apigw consumer id'), max_length=36, null=True, blank=True)

    @property
    def created_at(self):
        return self.date_joined

    @property
    def is_unsubscribed(self):
        return self.unsubscribed_at is not None

    @is_unsubscribed.setter
    def is_unsubscribed(self, value):
        self.unsubscribed_at = timezone.now()

    def set_password(self, raw_password):
        setattr(self, '__password_updated', True)
        return super(AbstractUserModel, self).set_password(raw_password)

    def __str__(self):
        return '{}:{}'.format(self.username, self.id)

    def serialized_as_string(self):
        serialized = ''
        for f in DJANGO_SSO_REQUIRED_USER_FIELDS + ('password',):
            field = getattr(self, f, None)
            if f == 'password':
                pass
                """
                if field and len(field):
                    print('PASS', f, field)
                    (algorithm, iterations, salt, hash) = field.split('$')
                    print('PASS', (algorithm, iterations, salt, hash))
                    serialized += hash
                else:
                    serialized += str(field)
                """
            else:
                serialized += str(field)


        # logger.debug('serialized_as_string {}'.format(serialized))
        return serialized

    def __init__(self, *args, **kwargs):
        super(AbstractUserModel, self).__init__(*args, **kwargs)
        setattr(self, '__serialized_as_string', self.serialized_as_string())

    def save(self, *args, **kwargs):
        if self._state.adding:
            if not self.first_name:
                self.first_name = ''
            if not self.last_name:
                self.last_name = ''
        return super(AbstractUserModel, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse("users:detail", args=[self.sso_id])

    def check_password(self, raw_password):
        # password is hashed, checking if will be runtime hardened
        # taken from django.contrib.auth.hashers.check_password
        old_hashed_password = self.password

        preferred = get_hasher('default')

        try:
            hasher = identify_hasher(old_hashed_password)
        except ValueError:
            hasher = None

        if hasher is not None:
            hasher_changed = hasher.algorithm != preferred.algorithm
            if hasher_changed:
                logger.info(
                    '!!! Password hasher for {} changed from {} to {}'.format(self,
                                                                              hasher.algorithm,
                                                                              preferred.algorithm))
            must_update = hasher_changed or hasher.must_update(old_hashed_password)
            if must_update:
                setattr(self, '__password_hardened', True)

        return super(AbstractUserModel, self).check_password(raw_password)

    def update_rev(self, commit=False):
        logger.info('Updating rev for User {0}'.format(self))
        self.sso_rev = F('sso_rev') + 1
        if commit:
            self.save()
            self.refresh_from_db()

    def unsubscribe(self):
        logger.info('Unsubscribing User {0}'.format(self))
        try:
            with transaction.atomic():
                self.unsubscribed_at = timezone.now()
                self.is_active = False
                self.update_rev(True)

        except Exception as e:
            logger.error(
                'Unsubscription error {0} for User {1}'.format(e, self),
                exc_info=True)



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns

from .views import ProfileViewSet


_urlpatterns = [
    url(r'^$', ProfileViewSet.as_view({'get': 'list'}), name="list"),
    url(r'^(?P<sso_id>[-\w]+)/$', ProfileViewSet.as_view({'get': 'retrieve', 'patch': 'partial_update'}), name="detail"),
]

urlpatterns = format_suffix_patterns(_urlpatterns)



import datetime
import logging

import jwt
from django.conf import settings
from allauth.account.adapter import get_adapter

from .handlers import jwt_encode

logger = logging.getLogger('backend')


def set_cookie(response, key, value, days_expire = 7):
    if days_expire is None:
        max_age = 365 * 24 * 60 * 60  #one year
    else:
        max_age = days_expire * 24 * 60 * 60 
    expires = datetime.datetime.strftime(datetime.datetime.utcnow() + datetime.timedelta(seconds=max_age), "%a, %d-%b-%Y %H:%M:%S GMT")

    response.set_cookie(key, value, max_age=None, expires=expires, path='/', domain=settings.COOKIE_DOMAIN, secure=None, httponly=False)


def invalidate_cookie(response):
    response.set_cookie('jwt', None, max_age=None, expires='Thu, 01 Jan 1970 00:00:01 GMT', path='/', domain=settings.COOKIE_DOMAIN, secure=None, httponly=False)


def get_request_jwt(request):
    jwt_header = request.META.get('HTTP_AUTHORIZATION', None)
    if jwt_header is not None:
        jwt_header = jwt_header.replace('Bearer ', '')
        
    return request.COOKIES.get('jwt', jwt_header)


def update_response_jwt(received_jwt, jwt_user, request, response):
    unverified_payload = jwt.decode(received_jwt, None, False)
    jwt_fingerprint = unverified_payload['fp']
    adapter = get_adapter(request)

    logger.info('Updating response JWT for User {0} with fingerprint {1}'.format(request.user, jwt_fingerprint))

    # creates new actual_device
    actual_device = adapter.add_user_device(jwt_user, jwt_fingerprint)
    
    token = jwt_encode(jwt_user, actual_device)
    #max_age = 365 * 24 * 60 * 60  #one year
    #expires = datetime.datetime.strftime(datetime.datetime.utcnow() + datetime.timedelta(seconds=max_age), "%a, %d-%b-%Y %H:%M:%S GMT")

    set_cookie(response, 'jwt', token, None)


def get_request_jwt_fingerprint(request):
    received_jwt = get_request_jwt(request)
    if received_jwt is None:
        raise KeyError('No token specified')

    unverified_payload = jwt.decode(received_jwt, None, False)
    return unverified_payload.get('fp')



import logging
from django.contrib.auth import get_user_model
from django.db.models.signals import pre_delete, pre_save, post_save
from django.dispatch import receiver
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from allauth.account.adapter import get_adapter

from django_sso_app.core.apps.api_gateway.functions import delete_consumer
from django_sso_app.core.utils import get_profile_model

logger = logging.getLogger('users')
User = get_user_model()
Profile = get_profile_model()


@receiver(pre_save, sender=User)
def check_upated_fields(sender, instance, **kwargs):
    if not instance._state.adding:  # if instance.pk:
        profile_field_updated = hasattr(instance, '__profile_field_updated')
        email_updated = hasattr(instance, '__email_updated')
        subscription_updated = hasattr(instance, '__subscription_updated') or instance.is_unsubscribed
        password_updated = hasattr(instance, '__password_updated')
        field_updated = instance.__serialized_as_string != \
                        instance.serialized_as_string()

        update_rev = False

        if field_updated:
            logger.info('User {0} updated DJANGO_SSO_REQUIRED_USER_FIELDS \n{1}\n{2}'.format(instance, instance.__serialized_as_string, instance.serialized_as_string()))
            update_rev = True

        if profile_field_updated:
            logger.info('User {0} updated DJANGO_SSO_REQUIRED_PROFILE_FIELDS')
            update_rev = True

        if email_updated:
            logger.info('User {0} updated email'.format(instance))

        if password_updated:
            logger.info('User {0} updated password'.format(instance))

        if subscription_updated:
            logger.info('User {0} updated subscription'.format(instance))

        if email_updated or password_updated or field_updated or subscription_updated:
            # __password_hardened attribute set by
            # backend.serializers.LoginSerializer on runtime password hardening
            update_rev = True

        if hasattr(instance, '__password_hardened'):
            update_rev = False

        if update_rev:
            instance.update_rev()  # updating rev
            setattr(instance, '__rev_updated', True)

            logger.info('rev updated, removing all devices for user {}.'.format(instance))
            adapter = get_adapter()
            adapter.remove_all_user_devices(instance)


@receiver(post_save, sender=User)
def fetch_from_db_if_updated(sender, instance, created, **kwargs):
    try:
        instance.sso_app_profile
    except ObjectDoesNotExist:
        logger.info('User {0} has no profile, creating one.'.format(instance))
        Profile.objects.create(user=instance, sso_id=instance.sso_id, sso_rev=0)

    if not created:
        if hasattr(instance, '__rev_updated'):
            logger.info('User {0} updated rev, refreshing instance from DB'.format(instance))
            instance.refresh_from_db()


@receiver(pre_delete, sender=User)
def delete_apigw_user_consumer(sender, instance, **kwargs):
    logger.info('Deleting apigw Consumer for User {0}'.format(instance))
    if not settings.TESTING_MODE:
        status_code, r1 = delete_consumer(instance)
        logger.info('Kong consumer deleted ({0}), {1}'.format(status_code, r1))
        if status_code >= 300:
            if status_code != 404:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) Deleting apigw consumer for User {1}, {2}'.format(
                        status_code, instance, r1))
                raise Exception(
                    "Error deleting apigw consumer, {}".format(status_code))



from django.conf import settings
from django.conf.urls import url
from django.urls import include


sso_app_urlpatterns = []

if settings.DJANGO_SSO_BACKEND_ENABLED:
    from django_sso_app.core.apps.groups.urls import urlpatterns as groups_urls
    from django_sso_app.core.apps.users.urls import urlpatterns as users_urls
    from django_sso_app.core.apps.services.urls import urlpatterns as services_urls
    from django_sso_app.core.apps.passepartout.urls import urlpatterns as passepartout_urls

    sso_app_urlpatterns += [
        url(r'^api/v1/auth/groups/', include((groups_urls, 'groups'), namespace="group")),
        url(r'^api/v1/auth/users/', include((users_urls, 'users'), namespace="user")),
        url(r'^api/v1/services/', include((services_urls, 'services'), namespace="service")),
        url(r'^api/v1/passepartout/', include((passepartout_urls, 'passepartout'), namespace="passepartout")),
    ]

    if not settings.DJANGO_SSO_APP_ENABLED and settings.DJANGO_SSO_APP_HAS_CUSTOM_USER_PROFILE:
        from django_sso_app.core.apps.profiles.urls import urlpatterns as profiles_urls

        sso_app_urlpatterns += [
            url(r'^api/v1/profiles/', include((profiles_urls, 'profiles'), namespace="profile")),
        ]

elif settings.DJANGO_SSO_APP_ENABLED and not settings.DJANGO_SSO_APP_HAS_CUSTOM_USER_PROFILE:
    from django_sso_app.core.apps.profiles.urls import urlpatterns as profiles_urls

    sso_app_urlpatterns += [
        url(r'^api/v1/profiles/', include((profiles_urls, 'profiles'), namespace="profile")),
    ]



from allauth.account.forms import SignupForm as allauth_SignupForm
from django import forms
from django.utils.translation import ugettext_lazy as _

"""
ROLE_CHOICES = (
    (-1, 'Staff'),
    (0, 'Cityzen'),
    (1, 'Institutional'),
    (2, 'Professionist'),
    (4, 'Company'),
    (6, 'Istitutional Partition Item'),
    (7, 'App'),
)

DJANGO_SSO_REQUIRED_PROFILE_FIELDS = (
                  'email',
                  'password',
                  'role',
                  'first_name', 'last_name',
                  'description', 'picture', 'birthdate',
                  'latitude', 'longitude',
                  'country',
                  'address'
                  )
"""


class SignupForm(allauth_SignupForm):
    # role = forms.MultipleChoiceField(widget=forms.CheckboxSelectMultiple,
    #                                  choices=ROLE_CHOICES)

    first_name = forms.CharField(
        label=_('First name'),
        widget=forms.TextInput(
        attrs={'type': 'text',
               'placeholder': _('First name')}))

    last_name = forms.CharField(
        label=_('Last name'),
        widget=forms.TextInput(
        attrs={'type': 'text',
               'placeholder': _('Last name')}))

    description = forms.CharField(
        label=_('Description'),
        widget=forms.Textarea(
        attrs={'placeholder': _('Description')}))

    picture = forms.ImageField(
        label=_('Picture'),
    )

    birthdate = forms.DateField(
        label=_('Birthdate'),
        widget=forms.TextInput(
        attrs={ 'type': 'date',
                'placeholder': _('Birthdate')}))

    latitude = forms.CharField(
        label=_('Latitude'),
        widget=forms.TextInput(
        attrs={ 'type': 'text',
                'placeholder': _('Latitude')}))

    longitude = forms.CharField(
        label=_('Longitude'),
        widget=forms.TextInput(
        attrs={ 'type': 'text',
                'placeholder': _('Longitude')}))

    country = forms.CharField(
        label=_('Country'),
        widget=forms.TextInput(
        attrs={ 'type': 'text',
                'placeholder': _('Country')}))

    address = forms.CharField(
        label=_('Address'),
        widget=forms.Textarea(
        attrs={'placeholder': _('Address')}))

    language = forms.CharField(
        label=_('Language'),
        widget=forms.TextInput(
        attrs={'type': 'text',
             'placeholder': _('Language')}))

    def __init__(self, *args, **kwargs):
        super(SignupForm, self).__init__(*args, **kwargs)

        self.fields['birthdate'].required = False
        self.fields['picture'].required = False
        self.fields['description'].required = False
        self.fields['language'].required = False

    def get_cleaned_data(self):
        return self.clean()



from . import *


DJANGO_SSO_DEFAULT_CONSUMER_GROUP = env('DJANGO_SSO_DEFAULT_CONSUMER_GROUP', default="users")

DJANGO_SSO_CONSUMER_USERNAME_HEADER = env('DJANGO_SSO_CONSUMER_USERNAME_HEADER',
                                          default='HTTP_X_CONSUMER_USERNAME')
DJANGO_SSO_CONSUMER_GROUPS_HEADER = env('DJANGO_SSO_CONSUMER_GROUPS_HEADER',
                                        default='HTTP_X_CONSUMER_GROUPS')


_DJANGO_SSO_DEFAULT_FRONTEND_APP = 'django_sso_app.core.apps.frontend'
DJANGO_SSO_FRONTEND_APP = env('DJANGO_SSO_FRONTEND_APP', default=_DJANGO_SSO_DEFAULT_FRONTEND_APP)
DJANGO_SSO_HAS_CUSTOM_FRONTEND_APP = (DJANGO_SSO_FRONTEND_APP != _DJANGO_SSO_DEFAULT_FRONTEND_APP)


_DJANGO_SSO_APP_DEFAULT_USER_PROFILE = 'django_sso_app.backend.profiles'
DJANGO_SSO_APP_USER_PROFILE = env('DJANGO_SSO_APP_USER_PROFILE',
                                  default=_DJANGO_SSO_APP_DEFAULT_USER_PROFILE)  # patch
DJANGO_SSO_APP_USER_PROFILE_MODEL = env('DJANGO_SSO_APP_USER_PROFILE_MODEL',
                                        default=DJANGO_SSO_APP_USER_PROFILE + '.models.Profile')  # patch
DJANGO_SSO_APP_USES_EXTERNAL_USER_PROFILE = DJANGO_SSO_APP_USER_PROFILE is not None
DJANGO_SSO_APP_HAS_CUSTOM_USER_PROFILE = (DJANGO_SSO_APP_USER_PROFILE != _DJANGO_SSO_APP_DEFAULT_USER_PROFILE)
DJANGO_SSO_APP_SERVER = env.bool('DJANGO_SSO_APP_SERVER', default=False)


DJANGO_SSO_REQUIRED_USER_FIELDS = (
    'username',
    'email',
)

DJANGO_SSO_REQUIRED_PROFILE_FIELDS = (
    'role',
    'first_name', 'last_name',
    'description', 'picture', 'birthdate',
    'latitude', 'longitude',
    'country',
    'address',
    'language'
)



# Generated by Django 2.2.4 on 2019-09-13 18:24

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('devices', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Passepartout',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('deleted', models.BooleanField(default=False)),
                ('token', models.CharField(db_index=True, max_length=36, unique=True)),
                ('jwt', models.TextField()),
                ('bumps', models.PositiveIntegerField(default=0)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='devices.Device', verbose_name='device')),
            ],
            options={
                'unique_together': {('token', 'deleted')},
            },
        ),
    ]



# -*- coding: utf-8 -*-
"""
    Dummy conftest.py for django_sso_app.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""

# import pytest
# import os
# import sys

# os.environ['DJANGO_SETTINGS_MODULE'] = 'backend.settings.test'
# sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))



"""
Base settings to build other settings files upon.
"""

import os
import sys
import environ


env = environ.Env()
ROOT_DIR = (
    environ.Path(__file__) - 4
)  # (django_sso_app/backend/config/settings/common.py - 4 = django_sso_app/)
APPS_DIR = ROOT_DIR.path("backend")

gettext = lambda s: s


# django-sso-app
DJANGO_SSO_APP_SERVER = False
DJANGO_SSO_BACKEND_ENABLED = env.bool('DJANGO_SSO_BACKEND_ENABLED', default=True)
DJANGO_SSO_APP_ENABLED = env.bool('DJANGO_SSO_APP_ENABLED', default=(not DJANGO_SSO_BACKEND_ENABLED))

if DJANGO_SSO_BACKEND_ENABLED or DJANGO_SSO_APP_ENABLED:
    from django_sso_app.core.settings import environ, env
else:
    READ_DOT_ENV_FILE = env.bool("DJANGO_READ_DOT_ENV_FILE", default=False)
    if READ_DOT_ENV_FILE:
        # OS environment variables take precedence over variables from .env
        env.read_env(str(ROOT_DIR.path(".env")))

if DJANGO_SSO_BACKEND_ENABLED:
    from django_sso_app.core.settings.backend import *
    from django_sso_app.core.settings.app import *
elif DJANGO_SSO_APP_ENABLED:
    from django_sso_app.core.settings.app import *

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = env.bool("DJANGO_DEBUG", True)

# Local time zone. Choices are
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# though not all of them may be available with every OS.
# In Windows, this must be set to your system time zone.
TIME_ZONE = env("DJANGO_TIME_ZONE", default="UTC") # pai
# https://docs.djangoproject.com/en/dev/ref/settings/#language-code
LANGUAGE_CODE = env("DJANGO_LANGUAGE_CODE", default="en-us") # pai
# https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = env.int("DJANGO_SITE_ID", default=1) # pai
# https://docs.djangoproject.com/en/dev/ref/settings/#use-i18n
USE_I18N = True
# https://docs.djangoproject.com/en/dev/ref/settings/#use-l10n
USE_L10N = True
# https://docs.djangoproject.com/en/dev/ref/settings/#use-tz
USE_TZ = True
# https://docs.djangoproject.com/en/dev/ref/settings/#locale-paths
LOCALE_PATHS = [APPS_DIR.path("locale")]

# pai
TESTING_MODE = 'test' in sys.argv or DEBUG
DJANGO_ALLAUTH_ENABLED = env.bool('DJANGO_ALLAUTH_ENABLED', default=(not DJANGO_SSO_APP_ENABLED))

# DATABASES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
if DEBUG:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(ROOT_DIR, 'db.sqlite3'),
        }
    }
else:
    DATABASES = {
        "default": env.db("DATABASE_URL", default="postgres:///django_sso_app")
    }
    DATABASES["default"]["ATOMIC_REQUESTS"] = True

# URLS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#root-urlconf
ROOT_URLCONF = "django_sso_app.backend.config.urls.base"
# https://docs.djangoproject.com/en/dev/ref/settings/#wsgi-application
WSGI_APPLICATION = "django_sso_app.backend.config.wsgi.application"

# APPS
# ------------------------------------------------------------------------------
DJANGO_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # "django.contrib.humanize", # Handy template tags
    "django.contrib.admin",
]
THIRD_PARTY_APPS = [
    "crispy_forms",
    # pai
    "corsheaders",
]

if DJANGO_ALLAUTH_ENABLED or DJANGO_SSO_BACKEND_ENABLED:
    THIRD_PARTY_APPS += [
        "allauth",
        "allauth.account",
        "allauth.socialaccount",
    ]

THIRD_PARTY_APPS += [
    "rest_framework",
    "rest_framework.authtoken",
    "rest_framework_swagger",

    "django_filters",
    "django_celery_beat",
]

LOCAL_APPS = [
    # Your stuff: custom apps go here
    "django_sso_app.backend.users.apps.UsersConfig",
]

# django-sso-app
if DJANGO_SSO_BACKEND_ENABLED:
    LOCAL_APPS += [
        "rest_auth",
        "rest_auth.registration",
    ] + DJANGO_SSO_BACKEND_CORE_APPS
elif DJANGO_SSO_APP_ENABLED:
    LOCAL_APPS += DJANGO_SSO_APP_CORE_APPS


# https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

# MIGRATIONS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#migration-modules
# MIGRATION_MODULES = {"sites": "backend.contrib.sites.migrations"} # pai

# AUTHENTICATION
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#authentication-backends
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
]
if DJANGO_ALLAUTH_ENABLED or DJANGO_SSO_BACKEND_ENABLED:
    AUTHENTICATION_BACKENDS += [
        "allauth.account.auth_backends.AuthenticationBackend",
    ]

# https://docs.djangoproject.com/en/dev/ref/settings/#auth-user-model
AUTH_USER_MODEL = "users.User"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-redirect-url
LOGIN_REDIRECT_URL = "/profile/"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-url
LOGIN_URL = "/login/"

# PASSWORDS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#password-hashers
"""
PASSWORD_HASHERS = [
    # https://docs.djangoproject.com/en/dev/topics/auth/passwords/#using-argon2-with-django
    "django.contrib.auth.hashers.Argon2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
]"""
# django
"""
[
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
    'django.contrib.auth.hashers.SHA1PasswordHasher',
    'django.contrib.auth.hashers.MD5PasswordHasher',
    'django.contrib.auth.hashers.UnsaltedSHA1PasswordHasher',
    'django.contrib.auth.hashers.UnsaltedMD5PasswordHasher',
    'django.contrib.auth.hashers.CryptPasswordHasher',
]
"""
PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',

    'django.contrib.auth.hashers.SHA1PasswordHasher'
]

# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]

# MIDDLEWARE
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#middleware
MIDDLEWARE = [
    # pai
    "django_sso_app.backend.middleware.x_forwarded_for.middleware.XForwardedForMiddleware",

    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    # pai
    "corsheaders.middleware.CorsMiddleware",

    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
]
# pai
if DJANGO_SSO_APP_ENABLED:
    MIDDLEWARE.append('django_sso_app.app.middleware.DjangoSsoAppMiddleware')

MIDDLEWARE += [
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

# STATIC
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = str(ROOT_DIR("static"))
# https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = "/static/"
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = [str(APPS_DIR.path("static"))]
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

# MEDIA
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(ROOT_DIR("media"))
# https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/"

# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # https://docs.djangoproject.com/en/dev/ref/settings/#template-dirs
        "DIRS": [str(APPS_DIR.path("templates"))],
        "OPTIONS": {
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-loaders
            # https://docs.djangoproject.com/en/dev/ref/templates/api/#loader-types
            "loaders": [
                "django.template.loaders.filesystem.Loader",
                "django.template.loaders.app_directories.Loader",
            ],
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",

                # pai
                'django_sso_app.backend.context_processors.get_repository_rev',
                'django_sso_app.backend.context_processors.get_auth_settings',
            ],
        },
    }
]
# http://django-crispy-forms.readthedocs.io/en/latest/install.html#template-packs
CRISPY_TEMPLATE_PACK = "bootstrap4"

# FIXTURES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#fixture-dirs
FIXTURE_DIRS = (str(APPS_DIR.path("fixtures")),)

# SECURITY
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#session-cookie-httponly
SESSION_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#csrf-cookie-httponly
CSRF_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#secure-browser-xss-filter
SECURE_BROWSER_XSS_FILTER = True
# https://docs.djangoproject.com/en/dev/ref/settings/#x-frame-options
X_FRAME_OPTIONS = "DENY"

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = env(
    "DJANGO_EMAIL_BACKEND", default="django.core.mail.backends.smtp.EmailBackend"
)
# https://docs.djangoproject.com/en/2.2/ref/settings/#email-timeout
EMAIL_TIMEOUT = 5

# ADMIN
# ------------------------------------------------------------------------------
# Django Admin URL.
ADMIN_URL = "admin/"
# https://docs.djangoproject.com/en/dev/ref/settings/#admins
ADMINS = [("""Luca Bertuol""", "paiuolo@gmail.com")]
# https://docs.djangoproject.com/en/dev/ref/settings/#managers
MANAGERS = ADMINS

# LOGGING
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#logging
# See https://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.

from .logging import LOGGING

# Celery
# ------------------------------------------------------------------------------
if USE_TZ:
    # http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-timezone
    CELERY_TIMEZONE = TIME_ZONE
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-broker_url
CELERY_BROKER_URL = env("CELERY_BROKER_URL", default="redis://localhost:6379/0")
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-result_backend
CELERY_RESULT_BACKEND = CELERY_BROKER_URL
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-accept_content
CELERY_ACCEPT_CONTENT = ["json"]
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-task_serializer
CELERY_TASK_SERIALIZER = "json"
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-result_serializer
CELERY_RESULT_SERIALIZER = "json"
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-time-limit
# TODO: set to whatever value is adequate in your circumstances
CELERY_TASK_TIME_LIMIT = env("CELERY_TASK_TIME_LIMIT", default=(5 * 60))
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-soft-time-limit
# TODO: set to whatever value is adequate in your circumstances
CELERY_TASK_SOFT_TIME_LIMIT = env("CELERY_TASK_SOFT_TIME_LIMIT", default=60)
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#beat-scheduler
CELERY_BEAT_SCHEDULER = "django_celery_beat.schedulers:DatabaseScheduler"

# lang
LANGUAGES = (
    ## Customize this
    ('it', gettext('it')),
    ('en-gb', gettext('en')),
    ('es', gettext('es')),
    ('pt', gettext('pt')),
    ('fr', gettext('fr')),
    ('de', gettext('de')),
)

# pai
if DJANGO_ALLAUTH_ENABLED:
    # django-allauth
    # ------------------------------------------------------------------------------
    ACCOUNT_ALLOW_REGISTRATION = env.bool("DJANGO_ACCOUNT_ALLOW_REGISTRATION", True)
    # https://django-allauth.readthedocs.io/en/latest/configuration.html
    ACCOUNT_AUTHENTICATION_METHOD = "username_email" # pai
    # https://django-allauth.readthedocs.io/en/latest/configuration.html
    ACCOUNT_EMAIL_REQUIRED = True
    # https://django-allauth.readthedocs.io/en/latest/configuration.html
    ACCOUNT_EMAIL_VERIFICATION = "mandatory"
    # https://django-allauth.readthedocs.io/en/latest/configuration.html
    ACCOUNT_ADAPTER = "django_sso_app.backend.users.adapters.AccountAdapter"
    # https://django-allauth.readthedocs.io/en/latest/configuration.html
    SOCIALACCOUNT_ADAPTER = "django_sso_app.backend.users.adapters.SocialAccountAdapter"
    # pai
    ACCOUNT_CONFIRM_EMAIL_ON_GET = True
    ACCOUNT_USERNAME_REQUIRED = True
    ACCOUNT_AUTHENTICATED_LOGIN_REDIRECTS = False
    ACCOUNT_EMAIL_CONFIRMATION_ANONYMOUS_REDIRECT_URL = LOGIN_URL
    ACCOUNT_EMAIL_CONFIRMATION_AUTHENTICATED_REDIRECT_URL = LOGIN_URL
    ACCOUNT_SESSION_REMEMBER = False

if DJANGO_SSO_BACKEND_ENABLED:
    ACCOUNT_ADAPTER = "django_sso_app.backend.adapters.UserAdapter"
    ACCOUNT_FORMS = {
        'signup': 'django_sso_app.backend.forms.SignupForm'
    }


# Your stuff...

# pai

FILE_UPLOAD_PERMISSIONS = 0o644
FILE_UPLOAD_HANDLERS = [
    'django.core.files.uploadhandler.TemporaryFileUploadHandler',
]

ENABLE_HTTPS = env.bool("ENABLE_HTTPS", False)

ACCOUNT_DEFAULT_HTTP_PROTOCOL = env("ACCOUNT_DEFAULT_HTTP_PROTOCOL", default="https" if ENABLE_HTTPS else "http")
APPEND_SLASH = True

APP_DOMAIN = env("APP_DOMAIN", default="localhost:8000")
APP_DOMAINS = env.list("APP_DOMAINS", default=[APP_DOMAIN, ])
APP_URL = ACCOUNT_DEFAULT_HTTP_PROTOCOL + '://' + APP_DOMAIN

# context_processors
COOKIE_DOMAIN = env("COOKIE_DOMAIN", default=APP_DOMAIN)
REPOSITORY_REV = env("REPOSITORY_REV", default=None)
EMAILS_DOMAIN = env('EMAILS_DOMAIN', default=APP_DOMAIN) # domain name specified in email templates
EMAILS_SITE_NAME = env('EMAILS_SITE_NAME', default=EMAILS_DOMAIN) # site name specified in email templates

# cors
if DEBUG:
    CORS_ORIGIN_ALLOW_ALL = True
else:
    # https://github.com/ottoyiu/django-cors-headers
    # CORS headers defaults to 'accounts.example.com'
    _CORS_ORIGINS = env("CORS_ORIGINS", default='{0}://{1}'.format(ACCOUNT_DEFAULT_HTTP_PROTOCOL, COOKIE_DOMAIN))
    CORS_ORIGIN_WHITELIST = list(map(lambda x: '{}'.format(x.replace(' ', '')), _CORS_ORIGINS.split(',')))
    #CORS_ORIGIN_WHITELIST = _CORS_ORIGINS.split(',')
    CORS_ALLOW_CREDENTIALS = True

# csrf
# https://docs.djangoproject.com/en/2.0/ref/settings/#csrf-trusted-origins

if DEBUG:
    CSRF_COOKIE_DOMAIN = None
    CORS_ORIGIN_ALLOW_ALL = True
    CSRF_TRUSTED_ORIGINS = []
else:
    CSRF_COOKIE_DOMAIN = APP_DOMAIN
    CSRF_TRUSTED_ORIGINS = ['.{0}'.format(COOKIE_DOMAIN)]

# django-sso-app
if DJANGO_SSO_BACKEND_ENABLED or DJANGO_SSO_APP_ENABLED:
    AUTHENTICATION_BACKENDS = AUTHENTICATION_BACKENDS + [
        'django_sso_app.app.backends.DjangoSsoAppBackend',
    ]


# djangorestframework

DRF_DEFAULT_AUTHENTICATION_CLASSES = [
    # 'rest_framework.authentication.SessionAuthentication',
    'rest_framework.authentication.TokenAuthentication'
]

if DJANGO_SSO_BACKEND_ENABLED:
    DRF_DEFAULT_AUTHENTICATION_CLASSES += [
        'django_sso_app.backend.authentication.JWTAuthentication',
    ]
elif DJANGO_SSO_APP_ENABLED:
    DRF_DEFAULT_AUTHENTICATION_CLASSES += [
        'django_sso_app.app.authentication.DjangoSsoAppAuthentication',
    ]


REST_FRAMEWORK = {
    'DEFAULT_AUTHENTICATION_CLASSES': DRF_DEFAULT_AUTHENTICATION_CLASSES,
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.IsAuthenticated',
    ),
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'PAGE_SIZE': 100,
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',),

    'DATETIME_FORMAT': '%Y-%m-%dT%H:%M:%S%z',
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    # https://www.django-rest-framework.org/community/3.10-announcement/#continuing-to-use-coreapi
}



# swagger

SWAGGER_SETTINGS = {
    'LOGIN_URL': '/login/',
    'LOGOUT_URL': '/logout/',
    'SECURITY_DEFINITIONS': {
        'JWT': {
            'type': 'apiKey',
            'name': 'Authorization',
            'in': 'header',
            'description': 'Bearer xxx'
        },
        'token': {
            'type': 'apiKey',
            'name': 'Authorization',
            'in': 'header',
            'description': 'Token xxx'
        }
    }
}


if DJANGO_SSO_BACKEND_ENABLED:
    # rest_auth

    REST_USE_JWT = True
    REST_AUTH_SERIALIZERS = {
        'LOGIN_SERIALIZER': 'django_sso_app.backend.serializers.LoginSerializer',
        'PASSWORD_RESET_SERIALIZER': 'django_sso_app.backend.serializers.PasswordResetSerializer',
        'USER_DETAILS_SERIALIZER' : 'django_sso_app.core.apps.users.serializers.UserSerializer',
    }

    REST_AUTH_REGISTER_SERIALIZERS = {
        'REGISTER_SERIALIZER': 'django_sso_app.backend.serializers.RegisterSerializer',
    }

    JWT_AUTH = {
        'JWT_AUTH_COOKIE': 'jwt',

        'JWT_PAYLOAD_HANDLER': 'django_sso_app.backend.handlers.jwt_payload_handler',
        'JWT_AUTH_HEADER_PREFIX': 'Bearer',
        'JWT_VERIFY': False,
    }


    print('ROOT_DIR', ROOT_DIR, "INSTALLED_APPS", INSTALLED_APPS)


# ------------------------------------------------------------------------------
from .logging import LOGGING



import base64
from io import BytesIO

from PIL import Image
from django import forms
from django.contrib import admin
from django.core.files.uploadedfile import InMemoryUploadedFile

from .models import Service, Subscription, TOS


class ServiceAdminForm(forms.ModelForm):
    picture = forms.ImageField(required=False)

    def clean_picture(self):
        picture_file = self.cleaned_data['picture']

        if picture_file is not None and type(picture_file) == InMemoryUploadedFile:
            picture_bin = picture_file.file.read()
            picture_image = Image.open(BytesIO(picture_bin))
            picture_format = picture_image.format

            if picture_format not in ('PNG', 'JPEG'):
                raise forms.ValidationError('Image must be either PNG or JPEG')

            picture_image.thumbnail((100, 100), Image.ANTIALIAS)
            
            image = BytesIO()
            picture_image.save(image, format=picture_format)

            return "data:image/{};base64,".format(picture_format.lower()) + base64.b64encode(image.getvalue()).decode('utf-8')
        else:
            return None


class TOSInline(admin.TabularInline):
    model = TOS


class ServiceAdmin(admin.ModelAdmin):
    form = ServiceAdminForm
    inlines = [
        TOSInline,
    ]


admin.site.register(Service, ServiceAdmin)
admin.site.register(Subscription)



import logging

from django.db import models
from django.contrib.auth import get_user_model

from ...models import CreatedAtModel

logger = logging.getLogger('devices')
User = get_user_model()


class Device(CreatedAtModel):
    # deleting deletes jwt by delete_device_jwt signal!

    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        related_name="devices"
    )

    fingerprint = models.CharField(max_length=32)
    user_agent = models.CharField(max_length=32, null=True, blank=True)

    apigw_jwt_id = models.CharField(max_length=36, null=True, blank=True)
    apigw_jwt_key = models.CharField(max_length=32, null=True, blank=True)
    apigw_jwt_secret = models.CharField(max_length=32, null=True, blank=True)

    def __str__(self):
      return '{}:{}'.format(self.id, self.fingerprint)



import logging

from django.conf import settings
from django.db.models.signals import pre_delete
from django.dispatch import receiver

from .models import Device
from ..api_gateway.functions import delete_consumer_jwt

logger = logging.getLogger('devices')

# signals

@receiver(pre_delete, sender=Device)
def delete_device_jwt(sender, instance, **kwargs):
    logger.info('Deleting apigw JWT for Device {0}, jwt_id: {1}'.format(instance, instance.apigw_jwt_id))
    if not settings.TESTING_MODE:
        status_code, r1 = delete_consumer_jwt(instance.user, instance.apigw_jwt_id)
        logger.info('Kong JWT deleted ({0}), {1}'.format(status_code, r1))
        if status_code >= 300:
            if status_code != 404:
                # delete_apigw_consumer(username)
                logger.error('Error ({0}) Deleting apigw JWT for Device {1}, {2}'.format(status_code, instance, r1))
                raise Exception(
                    "Error deleting apigw consumer jwt, {}".format(status_code))



from django.conf import settings

if settings.DJANGO_SSO_BACKEND_ENABLED:

    import json
    import re
    from rest_framework import status
    from django.core import mail
    from django.test import TestCase, Client
    from django.urls import reverse
    from django.contrib.auth import get_user_model
    from allauth.account.models import (
        EmailAddress,
        EmailConfirmation,
        EmailConfirmationHMAC,
    )

    from django_sso_app.core.apps.services.models import Service

    # initialize the APIClient app
    User = get_user_model()


    class TestLogin(TestCase):
        def setUp(self):
            self.client = Client()

            email = "pippo@disney.com"
            self.user = User.objects.create_user(username="pippo", email=email, password="paperina")
            self.user_email = EmailAddress.objects.create(user=self.user,
                                                          email=email,
                                                          primary=True,
                                                          verified=True)

            staff_email = 'staff@example.com'
            self.staff_user = User.objects.create_user(username="staff", email=staff_email, password="staff", is_staff=True)
            self.staff_user_email = EmailAddress.objects.create(user=self.staff_user,
                                                          email=staff_email,
                                                          primary=True,
                                                          verified=True)
            self.valid_staff_login = {
                'email': staff_email,
                'password': 'staff',
                'fingerprint': '000000'
            }

            self.valid_email_login = {
                'email': email,
                'password': 'paperina',
                'fingerprint': '123456'
            }
            self.valid_username_login = {
                'username': 'pippo',
                'password': 'paperina',
                'fingerprint': '123456'
            }
            self.invalid_login = {
                'email': 'topolino@disney.com',
                'password': ''
            }


        def test_can_login_with_email(self):
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_email_login),
                content_type='application/json'
            )

            self.assertEqual(response.status_code, status.HTTP_200_OK)

        def test_can_login_with_username(self):
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_username_login),
                content_type='application/json'
            )

            self.assertEqual(response.status_code, status.HTTP_200_OK)

        def test_login_returns_jwt_cookie(self):
            response = self.client.post(
                reverse('rest_login'),
                data = json.dumps(self.valid_email_login),
                content_type='application/json'
            )

            cookies = response.client.cookies.items()

            # print('\nRISPOSTA', response.data, '\nCookies\n', cookies, '\n')

            has_jwt_cookie = False
            for (key, val) in cookies:
                if key == "jwt":
                    has_jwt_cookie = True
                    break

            self.assertTrue(has_jwt_cookie)
        """
        def test_migrated_can_login_without_updating_rev(self):
            self.client.logout()

            migrated_user_email = 'supercacca@supercacca.com'
            hashed_password = 'pbkdf2_sha256$15000$KCmb0T1BofB7$fV4zlS1mFUC8HYhrgnAtEfvFzB1pgnyDloidsDLYVag='
            migrated_user = User.objects.create(
                username = 'supercacca',
                email = migrated_user_email,
                password = hashed_password
            )
            migrated_user_email = EmailAddress.objects.create(user=migrated_user,
                                                               email=migrated_user_email,
                                                               primary=True,
                                                               verified=True)
            valid_migrated_user_login = {
                'username': 'supercacca@supercacca.com',
                'password': 'supercacca',
                'fingerprint': '123'
            }


            previus_rev = migrated_user.sso_rev

            response = self.client.post(
                reverse('rest_login'),
                data=json.dumps(valid_migrated_user_login),
                content_type='application/json'
            )
            print('RESP', response)
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            migrated_user.refresh_from_db()
            second_rev = migrated_user.sso_rev
            self.assertEqual(previus_rev, second_rev)
        """
        def test_staff_user_can_not_login(self):
            response = self.client.post(
                reverse('rest_login'),
                data=json.dumps(self.valid_staff_login),
                content_type='application/json'
            )

            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    class TestRegister(TestCase):
        def setUp(self):
            email = "pippo@disney.com"
            self.user = User.objects.create_user(username="pippo", email=email, password="paperina")
            self.user_email = EmailAddress.objects.create(user=self.user,
                                                          email=email,
                                                          primary=True,
                                                          verified=True)
            self.valid_registration = {
                'email': 'topolino@disney.com',
                'username': 'topolino',
                'password1': 'paperina',
                'password2': 'paperina',
                'first_name': 'Mickey',
                'last_name': 'Mouse',
                'country': 'Italia',
                'latitude': 12.345,
                'longitude': 54.321,
                'referrer': 'http://example.com'
            }

            self.invalid_registration = {
                'email': 'topolino@d',
                'username': ''
            }

            self.service = Service.objects.create(name='example', url='http://example.com')

        def test_can_register(self):
            response = self.client.post(
                reverse('rest_register'),
                data = json.dumps(self.valid_registration),
                content_type='application/json'
            )

            self.assertEqual(response.status_code, status.HTTP_201_CREATED)

            new_user = User.objects.filter(email=self.valid_registration['email']).first()

            self.assertEqual(new_user.email, self.valid_registration['email'])

            email_address = EmailAddress.objects.filter(user=new_user, email=self.valid_registration['email']).first()

            self.assertNotEqual(email_address, None)

        def test_cannot_register_on_invalid_registration_data(self):
            response = self.client.post(
                reverse('rest_register'),
                data = json.dumps(self.invalid_registration),
                content_type='application/json'
            )

            self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        def test_registration_sends_confirmation_email(self):
            response = self.client.post(
                reverse('rest_register'),
                data = json.dumps(self.valid_registration),
                content_type='application/json'
            )

            self.assertEqual(len(mail.outbox), 1)



import logging

import requests
from django.conf import settings

logger = logging.getLogger('api_gateway')


def get_consumer_id(user):
    if hasattr(user, 'sso_id'):
        consumer_id = str(user.sso_id).replace('-', '_')
    else:
        consumer_id = str(user.sso_app_profile.sso_id).replace('-', '_')

    return consumer_id


def create_consumer(user):
    consumer_id = get_consumer_id(user)
    logger.info('creating apigw consumer for {}'.format(user.username))

    url = settings.APIGW_HOST + "/consumers/"
    data = {'username': consumer_id}

    r = requests.post(url, json=data)

    response_body = None
    status_code = r.status_code
    try:
        response_body = r.json()
    except:
        logger.info('Response ({}) has no payload'.format(status_code))

    logger.info('apigw response ({0}) {1}'.format(status_code, response_body))
    return status_code, response_body


def delete_consumer(user):
    consumer_id = get_consumer_id(user)
    logger.info('deleting apigw consumer for {}'.format(user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id

    r = requests.delete(url)

    response_body = None
    status_code = r.status_code
    try:
        response_body = r.json()
    except:
        logger.info('Response ({}) has no payload'.format(status_code))

    logger.info('apigw response ({0}) {1}'.format(status_code, response_body))
    return status_code, response_body


def create_consumer_jwt(user):
    consumer_id = get_consumer_id(user)
    logger.info('creating apigw consumer jwt for {}'.format(user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id + "/jwt/"
    data = {}

    r = requests.post(url, json=data)

    response_body = None
    status_code = r.status_code
    try:
        response_body = r.json()
    except:
        logger.info('Response ({}) has no payload'.format(status_code))

    logger.info('apigw response ({0}) {1}'.format(status_code, response_body))
    return status_code, response_body


def delete_consumer_jwt(user, jwt_id):
    consumer_id = get_consumer_id(user)
    logger.info('deleting apigw consumer jwt for {}'.format(user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id + "/jwt/" + jwt_id

    r = requests.delete(url)

    status_code = r.status_code

    logger.info('apigw response ({0}) {1}'.format(status_code, None))
    return r.status_code, None


def get_consumer_jwts(user):
    consumer_id = get_consumer_id(user)
    logger.info('getting apigw consumer jwts for {}'.format(user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id + "/jwt/"

    r = requests.get(url)

    response_body = None
    status_code = r.status_code
    try:
        response_body = r.json()
    except:
        logger.info('Response ({}) has no payload'.format(status_code))

    logger.info('apigw response ({0}) {1}'.format(status_code, response_body))
    return status_code, response_body


def create_consumer_acl(user, group_name=settings.DJANGO_SSO_DEFAULT_CONSUMER_GROUP):
    consumer_id = get_consumer_id(user)
    logger.info('creating apigw consumer acl {} for {}'.format(group_name,
                                                               user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id + "/acls/"
    data = {"group": group_name}

    r = requests.post(url, json=data)

    response_body = None
    status_code = r.status_code
    try:
        response_body = r.json()
    except:
        logger.info('Response ({}) has no payload'.format(status_code))

    logger.info('apigw response ({0}) {1}'.format(status_code, response_body))
    return status_code, response_body


def delete_consumer_acl(user, group_name):
    consumer_id = get_consumer_id(user)
    logger.info('deleting apigw consumer acl {} for {}'.format(group_name,
                                                               user.username))

    url = settings.APIGW_HOST + "/consumers/" + consumer_id + "/acls/" + group_name

    r = requests.delete(url)

    status_code = r.status_code

    logger.info('apigw response ({0}) {1}'.format(status_code, None))
    return r.status_code, None



import logging

from django.conf import settings
from rest_framework_jwt.authentication import JSONWebTokenAuthentication


logger = logging.getLogger('backend.authentication')

username_header = getattr(settings, 'DJANGO_SSO_CONSUMER_HEADER', 'HTTP_X_CONSUMER_USERNAME')
anonymous_username = getattr(settings, 'DJANGO_SSO_ANONYMOUS_USERNAME', 'anonymous')


class JWTAuthentication(JSONWebTokenAuthentication):
    def authenticate(self, request):
        username = request.META.get(username_header, None)

        if username == anonymous_username:
            logger.info('JWTAuthentication found Anonymous username header set. Skipping authentication.')
            return None
        else:
            return super(JWTAuthentication, self).authenticate(request)



from django.utils.deprecation import MiddlewareMixin

class XForwardedForMiddleware(MiddlewareMixin):
    def process_request(self, request):
        if 'HTTP_X_FORWARDED_FOR' in request.META:
            request.META['REMOTE_ADDR'] = request.META['HTTP_X_FORWARDED_FOR'].split(",")[0].strip()
        return None



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns

from .views import (passepartout_login_view, passepartout_logout_view)

_urlpatterns = [

    url(r'^login/(?P<token>\w+)/$',
        passepartout_login_view,
        name='passepartout_login'),
    
    url(r'^logout/(?P<device_id>\d+)/$',
        passepartout_logout_view,
        name='passepartout_logout')
]

urlpatterns = format_suffix_patterns(_urlpatterns)



import logging
import uuid

from django.utils.translation import activate as activate_translation
from django.conf import settings
from django.urls import reverse
from django.utils import timezone
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist

from allauth.account.adapter import DefaultAccountAdapter

from django_sso_app.core.apps.devices.models import Device
from django_sso_app.core.apps.services.models import Service, Subscription
from django_sso_app.core.utils import get_random_string, get_profile_model
from django_sso_app.backend.utils import get_request_jwt_fingerprint
from django_sso_app.core.apps.api_gateway.functions import create_consumer_jwt, create_consumer, create_consumer_acl

logger = logging.getLogger('backend')
Profile = get_profile_model()


class UserAdapter(DefaultAccountAdapter):

    def update_user_profile(self, user, cleaned_data, commit=True):
        logger.info(
            'Updating profile fields for {0}: {1}'.format(user, cleaned_data))

        profile = user.sso_app_profile
        if profile is None:
            logger.info('No profile.!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')

        for f in settings.DJANGO_SSO_REQUIRED_PROFILE_FIELDS:
            data = cleaned_data.get(f, None)
            logger.info('SAVING profile FIELD: {}. Data: {}'.format(f, data))
            setattr(profile, f, data)

        if commit:
            profile.save()

        return profile

    def new_user(self, request):
        """
        Instantiates a new User instance.
        """
        new_user_sso_id = uuid.uuid4()
        new_user = get_user_model()(sso_id=new_user_sso_id)

        return new_user

    def save_user(self, request, user, form, commit=True):
        """
        Saves a new `User` instance using information provided in the
        signup form.
        """

        cleaned_data = form.get_cleaned_data()
        logger.info(
            'Saving new user {0} with email {1} from form {2} with cleaned_data {3}'.format(cleaned_data.get('username'), cleaned_data.get('email'), form.__class__, cleaned_data))

        # !atomic
        new_user = super(UserAdapter, self).save_user(
            request, user, form, False)

        has_profile = True
        try:
            profile = new_user.sso_app_profile
        except ObjectDoesNotExist:
            has_profile = False

        if not has_profile:
            new_user.save()

        for f in ('email', 'username', 'is_unsubscribed'):
            data = cleaned_data.get(f, None)
            logger.info('SAVING USER FIELD: {}. Data: {}'.format(f, data))
            setattr(new_user, f, data)

        self.update_user_profile(user, cleaned_data, commit or has_profile)

        if commit:
            new_user.save()
        # !endatomic

        return new_user

    def render_mail(self, template_prefix, email, context):
        user = context.get('user')
        user_language = user.sso_app_profile.language
        logger.info('Rendering mail for {0} with language {1}'.format(user, user_language))
        activate_translation(user_language)
        context.update({
            'EMAILS_DOMAIN': settings.EMAILS_DOMAIN,
            'EMAILS_SITE_NAME': settings.EMAILS_SITE_NAME
        })
        return super(UserAdapter, self).render_mail(template_prefix, email, context)

    def get_email_confirmation_url(self, request, emailconfirmation):
        """Constructs the email confirmation (activation) url.
        Note that if you have architected your system such that email
        confirmations are sent outside of the request context `request`
        can be `None` here.
        """
        url = reverse(
            "account_confirm_email",
            args=[emailconfirmation.key])
        ret = settings.ACCOUNT_DEFAULT_HTTP_PROTOCOL + "://" + settings.EMAILS_DOMAIN + url
        return ret

    # pai

    def add_user_device(self, user, fingerprint, user_agent=None):
        logger.info(
            'Adding User Device for user {0} with fingerprint {1}'.format(user,
                                                                          fingerprint))

        device = Device(user=user, fingerprint=fingerprint)

        if not settings.TESTING_MODE:
            status_code, r1 = create_consumer_jwt(self)
            if status_code != 201:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) creating apigw consumer JWT, {1}'.format(
                        status_code, r1))
                raise Exception(
                    "Error creating apigw consumer jwt, {}".format(status_code))

            device.apigw_jwt_id = r1.get('id')
            device.apigw_jwt_key = r1.get('key')
            device.apigw_jwt_secret = r1.get('secret')

        else:
            device.apigw_jwt_id = get_random_string(36)
            device.apigw_jwt_key = get_random_string(32)
            device.apigw_jwt_secret = get_random_string(32)

        device.save()

        return device

    def remove_user_device(self, device):
        logger.info('Removing Device {0}'.format(device.id))
        device.delete()
        return 1

    def remove_all_user_devices(self, user):
        logger.info('Removing All User Devices for {0}'.format(user))
        removed = 0
        for d in user.devices.all():
            removed += self.remove_user_device(d)
        return removed

    def subscribe_to_service(self, user, referrer, update_rev=False, commit=True):
        logger.info('Subscribinig {0} to service {1}'.format(user, referrer))

        service = Service.objects.get(url=referrer)

        subscription, _subscription_created = \
            Subscription.objects.get_or_create(
                user=user, service=service)

        if _subscription_created:
            logger.info('Created service subscrption for {0}'.format(user))
            if update_rev:
                setattr(user, '__subscription_updated', True)
                user.update_rev(commit)
        else:
            logger.info(
                'User {0} already subscribed to {1}'.format(user, service))

        return _subscription_created

    def unsubscribe_from_service(self, user, subscription, update_rev=False,
                                 commit=True):
        logger.info('Unubscribinig {0} form service {1}'.format(user,
                                                                subscription.service))
        setattr(user, '__subscription_updated', True)

        subscription.unsubscribed_at = timezone.now()
        subscription.save()

        if update_rev:
            user.update_rev(commit)

    def create_apigw_consumer(self, user):
        logger.info('Adding apigw consumer for User {0}'.format(user))
        if not settings.TESTING_MODE:
            status_code_1, consumer = create_consumer(user)
            if status_code_1 != 201:
                logger.error('Error ({0}) creating apigw consumer, {1}'.format(
                    status_code_1, consumer))
                raise Exception(
                    "Error ({0}) creating apigw consumer, {1}".format(
                        status_code_1, consumer))

            status_code_2, acl = create_consumer_acl(user)
            if status_code_2 != 201:
                # delete_apigw_consumer(username)
                logger.error(
                    'Error ({0}) creating apigw consumer acl, {1}'.format(
                        status_code_2, acl))
                raise Exception(
                    "Error {0} creating apigw consumer acl, {1}".format(
                        status_code_2, acl))
            user.apigw_consumer_id = consumer['id']

        else:
            user.apigw_consumer_id = get_random_string(36)

        user.save()

    def get_request_device(self, request):
        received_jwt = get_request_jwt_fingerprint(request)
        return Device.objects.filter(fingerprint=received_jwt).first()

    def delete_request_device(self, request):
        removing_device = self.get_request_device(request)
        if removing_device is not None:
            logger.info('Removing logged out user device {0} for User {1}'.format(removing_device, request.user))
            self.remove_user_device(removing_device)
        else:
            logger.warning('User {0} is logging out WITHOUT removing device.'.format(request.user))




import logging
from urllib.parse import urlparse, urlencode

from django.conf import settings
from django.http import Http404
from django.shortcuts import redirect

from ..devices.models import Device
from ..services.models import Service
from django_sso_app.backend.utils import set_cookie, invalidate_cookie

from .models import Passepartout

logger = logging.getLogger('passepartout')


def get_base_url(full_url):
    parsed_url = urlparse(full_url)
    return_url = parsed_url.scheme + '://' + parsed_url.hostname
    if parsed_url.port is not None:
        return_url += ':' + str(parsed_url.port)
    logger.info('GETTING BASE URL for {0}: {1}'.format(full_url, return_url))
    return return_url


def get_referrer(request):
    referrer = request.META.get('HTTP_REFERER', None)
    if referrer is None:
        _index_previous_should_be = settings.DJANGO_SSO_DOMAINS_DICT[settings.APP_DOMAIN] - 1
        if _index_previous_should_be < 0:
            _last_sso_instance = settings.DJANGO_SSO_DOMAINS[-1]
            if _last_sso_instance == settings.APP_DOMAIN:
                raise Exception('Passepartout Referrer not set')
            else:
                referrer = settings.DJANGO_SSO_FULL_URLS_CHAIN[-1]
                logger.warning('Referrer was empty, setting as LAST SSO instance: {}'.format(referrer))
        else:
            referrer = settings.DJANGO_SSO_FULL_URLS_CHAIN[_index_previous_should_be]
            logger.warning('Referrer was empty, setting as PREVIOUS SSO instance: {}'.format(referrer))
    else:
        logger.debug('COMES FROM referrer: {}'.format(referrer))
    return referrer


def passepartout_login_view(request, token):
    """
    Receives unguessable token by path parameter and destination url by get parameter.
    1) checks user subscription to destination service (obtained by next param as Service url)
    2) checks its position in DJANGO_SSO_URLS ordered dictionary
    3) redirects to next APP_URL if there are subsquent sso domains or directly to service url (if redirecting to APP_URL request will include a jwt token COOKIES)
    """
    try:
        nextUrl = request.GET.get('next', None)
        logger.debug('ENTERING passepartout with next {}'.format(nextUrl))

        if nextUrl is None:
            raise Http404("No next url")

        try:
            passepartout = Passepartout.objects.get(deleted=False, token=token)
        except Passepartout.DoesNotExist:
            raise Http404("Passepartout not found")

        user = passepartout.device.user
        
        next_url_base_path = get_base_url(nextUrl)
        
        service = None
        if next_url_base_path not in settings.DJANGO_SSO_FULL_URLS_CHAIN:
            try:
                service = Service.objects.get(url=next_url_base_path)
            except Service.DoesNotExist:
                raise Http404("Service not found")

        referrer = get_referrer(request)

        comes_from = get_base_url(referrer)
        sso_urls_chain = settings.DJANGO_SSO_URLS_CHAIN
        sso_urls_chain_dict = settings.DJANGO_SSO_URLS_CHAIN_DICT
        logger.info('Passepartout chain dict: {0}, this is: {1}. Comes from {2} with nextUrl: {3}'.format(sso_urls_chain, settings.APP_URL, comes_from, nextUrl))
        
        previous_sso_instance_index = sso_urls_chain_dict[comes_from]
        last_sso_instance_index = len(sso_urls_chain) - 1
        next_sso_instance_index = None

        is_last_bump = previous_sso_instance_index == last_sso_instance_index
        goes_to = None
        redirect_args = None
        
        if is_last_bump:
            logger.info('Is last login bump')
            passepartout.deleted = True
            passepartout.save()
            goes_to = nextUrl
        else:
            redirect_args = {
                'next': nextUrl
            }
            next_sso_instance_index = last_sso_instance_index + 1
            goes_to = sso_urls_chain[next_sso_instance_index]
        
        #if subscribed is None:
        #    subscrition_created = user.subscribe_to_service(service)
        #    logger.info('Did passepartout subscribed user, {0} to {2}? {3}'.format(user, nextUrl, subscription_created))
        #    redirect_args['subscribed'] = 'true'

        formatted_redirect_args = ''
        if not is_last_bump:
            formatted_redirect_args = '?' + urlencode(redirect_args)

        redirect_to = goes_to + formatted_redirect_args
        logger.info('Passepartout login redirecting to {0}'.format(redirect_to))

        response = redirect(redirect_to)

        jwt_token = passepartout.jwt
        set_cookie(response, 'jwt', jwt_token, None)
    except Exception as e:
        logger.error('Passepartout login ERROR: {}'.format(e), exc_info=True)
        raise

    return response


def passepartout_logout_view(request, device_id):
    """
    Logout from all sso_urls_chain, can be called from protected service.
    """
    try:
        nextUrl = request.GET.get('next', None)
        if nextUrl is None:
            raise Http404("No next url")

        referrer = get_referrer(request)

        comes_from = get_base_url(referrer)
        sso_urls_chain = settings.DJANGO_SSO_URLS_CHAIN
        sso_urls_chain_dict = settings.DJANGO_SSO_URLS_CHAIN_DICT
        logger.info('Passepartout chain dict: {0}, this is: {1}. Comes from {2}'.format(sso_urls_chain, settings.APP_URL, comes_from))
        
        last_sso_instance_index = len(sso_urls_chain) - 1
        next_sso_instance_index = 0
        comes_from_sso_instance = False
        previous_sso_instance_index = 0
        is_last_bump = False
        
        try:
            previous_sso_instance_index = sso_urls_chain_dict[comes_from]
            next_sso_instance_index = previous_sso_instance_index + 1
            comes_from_sso_instance = True
            logger.info('Comes from sso instance, previous index: {0}, next index {1}'.format(previous_sso_instance_index, next_sso_instance_index))
        except KeyError:
            logger.info('Comes from protected service')
            pass
        
        # if coming from sso instance looks for next instances or is_last_bump
        if comes_from_sso_instance:
            is_last_bump = next_sso_instance_index >= last_sso_instance_index
        # if coming from protexted service looks for next sso instances or is_last_bump
        else:
            is_last_bump = next_sso_instance_index == last_sso_instance_index

        goes_to = None
        redirect_args = None
        
        if is_last_bump:
            goes_to = nextUrl
            logger.info('Is last logout bump, goes to {}'.format(goes_to))
        else:
            redirect_args = {
                'next': nextUrl
            }
            next_sso_instance_index = last_sso_instance_index + 1
            goes_to = sso_urls_chain[next_sso_instance_index]
            logger.info('Is median logout bump, goes to {}'.format(goes_to))

        formatted_redirect_args = ''
        if not is_last_bump:
            formatted_redirect_args = '?' + urlencode(redirect_args)

        redirect_to = goes_to + formatted_redirect_args
        logger.info('Passepartout logout redirecting to {0}'.format(redirect_to))

        response = redirect(redirect_to)

        if is_last_bump:
            try:
                device = Device.objects.get(id=device_id)
            except Device.DoesNotExist:
                raise Http404("Device not found")
            logger.info('Passepartout deleting device {0} on last bump.'.format(device))
            device.delete()

        invalidate_cookie(response)

    except Exception as e:
        logger.error('Passepartout logout ERROR: {}'.format(e), exc_info=True)
        raise

    return response




from .common import *


DJANGO_SSO_COOKIE_DOMAIN = env('DJANGO_COOKIE_DOMAIN', default='example.com')

DJANGO_SSO_DEACTIVATE_USER_ON_UNSUBSCRIPTION = env.bool(
    'DJANGO_SSO_DEACTIVATE_USER_ON_UNSUBSCRIPTION', False)
DJANGO_SSO_SERVICE_SUBSCRIPTION_IS_MANDATORY = env.bool(
    'DJANGO_SSO_SERVICE_SUBSCRIPTION_IS_MANDATORY', False)
DJANGO_SSO_STAFF_JWT = env('DJANGO_SSO_STAFF_JWT', default=None)

DJANGO_SSO_URL = env('DJANGO_SSO_URL', default='https://accounts.example.com')
DJANGO_SSO_BACKEND_URL = env('DJANGO_SSO_BACKEND_URL', default=DJANGO_SSO_URL)
DJANGO_SSO_SERVICE_URL = env('DJANGO_SSO_SERVICE_URL',
                             default='https://protectedservice.example.com')

DJANGO_SSO_LOGIN_URL = DJANGO_SSO_URL + '/login/'
DJANGO_SSO_REGISTER_URL = DJANGO_SSO_URL + '/signup/'
DJANGO_SSO_PASSWORD_RESET_URL = DJANGO_SSO_URL + '/send-password-reset/'

DJANGO_SSO_API_URL = DJANGO_SSO_URL + '/api/v1'
DJANGO_SSO_BACKEND_API_URL = DJANGO_SSO_BACKEND_URL + '/api/v1'

DJANGO_SSO_LOGOUT_URL = DJANGO_SSO_API_URL + '/auth/logout/'

DJANGO_SSO_USERS_URL = DJANGO_SSO_API_URL + '/profiles/users/'
DJANGO_SSO_BACKEND_USERS_URL = DJANGO_SSO_BACKEND_API_URL + '/profiles/users/'

DJANGO_SSO_USER_URL = DJANGO_SSO_USERS_URL + '{user_id}/'
DJANGO_SSO_BACKEND_USER_URL = DJANGO_SSO_BACKEND_USERS_URL + '{user_id}/'

DJANGO_SSO_USERS_CHECK_URL = DJANGO_SSO_USERS_URL + 'check/'
DJANGO_SSO_BACKEND_USERS_CHECK_URL = DJANGO_SSO_BACKEND_USERS_URL + 'check/'

DJANGO_SSO_USER_SUBSCRIPTIONS_URL = DJANGO_SSO_API_URL + '/profiles/users/{' \
                                                         'user_id}/subscriptions/'
DJANGO_SSO_BACKEND_USER_SUBSCRIPTIONS_URL = DJANGO_SSO_BACKEND_API_URL + \
                                            '/profiles/users/{' \
                                            'user_id}/subscriptions/'

DJANGO_SSO_USER_SUBSCRIPTIONS_CREATE_URL = DJANGO_SSO_USER_SUBSCRIPTIONS_URL \
                                           + 'create/{service_id}/'
DJANGO_SSO_BACKEND_USER_SUBSCRIPTIONS_CREATE_URL = \
    DJANGO_SSO_BACKEND_USER_SUBSCRIPTIONS_URL + 'create/{service_id}/'

DJANGO_SSO_USER_SUBSCRIPTION_UNSUBSCRIBE_URL = \
    DJANGO_SSO_USER_SUBSCRIPTIONS_URL + '{subscription_id}/unsubscribe/'
DJANGO_SSO_BACKEND_USER_SUBSCRIPTION_UNSUBSCRIBE_URL = \
    DJANGO_SSO_BACKEND_USER_SUBSCRIPTIONS_URL + '{subscription_id}/unsubscribe/'

DJANGO_SSO_SERVICES_URL = DJANGO_SSO_API_URL + '/services/'
DJANGO_SSO_BACKEND_SERVICES_URL = DJANGO_SSO_BACKEND_API_URL + '/services/'

DJANGO_SSO_SERVICE_SUBSCRIBE_URL = DJANGO_SSO_SERVICES_URL + '{service_id}/subscribe/'
DJANGO_SSO_BACKEND_SERVICE_SUBSCRIBE_URL = DJANGO_SSO_BACKEND_SERVICES_URL + \
                                           '{service_id}/subscribe/'

DJANGO_SSO_ADMIN_USER_URL = DJANGO_SSO_URL + '/admin/profiles/user/'

DJANGO_SSO_REPLICATE_PROFILE = env.bool('DJANGO_SSO_REPLICATE_PROFILE', False)
DJANGO_SSO_MANAGE_USER_GROUPS = env.bool('DJANGO_SSO_MANAGE_USER_GROUPS', True)

DJANGO_SSO_JWT_HEADER = env('DJANGO_SSO_JWT_HEADER', default='jwt')

DJANGO_SSO_ANONYMOUS_CONSUMER_USERNAME = env(
    'DJANGO_SSO_ANONYMOUS_CONSUMER_USERNAME', default="anonymous")

DJANGO_SSO_ANONYMOUS_CONSUMER_HEADER = env(
    'DJANGO_SSO_ANONYMOUS_CONSUMER_HEADER', default='HTTP_X_ANONYMOUS_COMSUMER')

DJANGO_SSO_APP_CORE_APPS = [
    'django_sso_app.core.apps.groups',

    DJANGO_SSO_APP_USER_PROFILE,
    DJANGO_SSO_FRONTEND_APP
]



from django.conf import settings
from django.urls import reverse


if settings.DJANGO_SSO_BACKEND_ENABLED:
    from django_sso_app.core.apps.users.models import AbstractUserModel

    class User(AbstractUserModel):
        pass

else:
    from django.contrib.auth.models import AbstractUser

    class User(AbstractUser):
        def get_absolute_url(self):
            return reverse("users:detail", kwargs={"username": self.username})



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns

from .views import GroupViewSet


urlpatterns = [
    url(r'^$', GroupViewSet.as_view({'get': 'list'}), name="list"),
    url(r'^(?P<pk>\w+)/$', GroupViewSet.as_view({'get': 'retrieve'}), name="detail"),
]

urlpatterns = format_suffix_patterns(urlpatterns)



from django_sso_app.core.apps.profiles.serializers import ProfilePublicSerializer as DjangoSSoAppProfilePublicSerializer
from django_sso_app.core.apps.profiles.serializers import ProfileSerializer as DjangoSSoAppProfileSerializer


class ProfileSerializer(DjangoSSoAppProfileSerializer):
    pass


class ProfilePublicSerializer(DjangoSSoAppProfilePublicSerializer):
    pass



from .base import *  # noqa
from .base import env

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = True
# https://docs.djangoproject.com/en/dev/ref/settings/#secret-key
SECRET_KEY = env(
    "DJANGO_SECRET_KEY",
    default="7MA0D3xJcONLvhB2ymIgkpc9Rulc9nt3hLStawwMZVAm6rfmW3wuoZvp2efwcfsy",
)
# https://docs.djangoproject.com/en/dev/ref/settings/#allowed-hosts
ALLOWED_HOSTS = ["localhost", "0.0.0.0", "127.0.0.1"]

# CACHES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#caches
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "",
    }
}

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = env(
    "DJANGO_EMAIL_BACKEND", default="django.core.mail.backends.console.EmailBackend"
)
# https://docs.djangoproject.com/en/dev/ref/settings/#email-host
EMAIL_HOST = "localhost"
# https://docs.djangoproject.com/en/dev/ref/settings/#email-port
EMAIL_PORT = 1025

# django-debug-toolbar
# ------------------------------------------------------------------------------
# https://django-debug-toolbar.readthedocs.io/en/latest/installation.html#prerequisites
INSTALLED_APPS += ["debug_toolbar"]  # noqa F405
# https://django-debug-toolbar.readthedocs.io/en/latest/installation.html#middleware
MIDDLEWARE += ["debug_toolbar.middleware.DebugToolbarMiddleware"]  # noqa F405
# https://django-debug-toolbar.readthedocs.io/en/latest/configuration.html#debug-toolbar-config
DEBUG_TOOLBAR_CONFIG = {
    "DISABLE_PANELS": ["debug_toolbar.panels.redirects.RedirectsPanel"],
    "SHOW_TEMPLATE_CONTEXT": True,
}
# https://django-debug-toolbar.readthedocs.io/en/latest/installation.html#internal-ips
INTERNAL_IPS = ["127.0.0.1", "10.0.2.2"]


# django-extensions
# ------------------------------------------------------------------------------
# https://django-extensions.readthedocs.io/en/latest/installation_instructions.html#configuration
INSTALLED_APPS += ["django_extensions"]  # noqa F405
# Celery
# ------------------------------------------------------------------------------
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-always-eager
CELERY_TASK_ALWAYS_EAGER = True
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#task-eager-propagates
CELERY_TASK_EAGER_PROPAGATES = True
# Your stuff...
# ------------------------------------------------------------------------------



from django.contrib import admin
from django.contrib.auth import admin as auth_admin
from django.contrib.auth import get_user_model
from django.conf import settings

from .forms import UserChangeForm, UserCreationForm

User = get_user_model()


if settings.DJANGO_SSO_BACKEND_ENABLED or settings.DJANGO_SSO_APP_ENABLED:
    from django_sso_app.core.utils import get_profile_model

    if settings.DJANGO_SSO_BACKEND_ENABLED:
        _list_display = ["username", "email", "is_superuser", "sso_id", "sso_rev"]
        _search_fields = ["username", "email", "sso_id"]
    elif settings.DJANGO_SSO_APP_ENABLED:
        _list_display = ["username", "email", "is_superuser"]
        _search_fields = ["username", "email"]

    Profile = get_profile_model()

    class ProfileInline(admin.StackedInline):
        model = Profile
        max_num = 1
        can_delete = False

        def get_readonly_fields(self, request, obj=None):
            # if obj is None:
            return ['sso_id', 'sso_rev']


    @admin.register(User)
    class UserAdmin(auth_admin.UserAdmin):
        form = UserChangeForm
        add_form = UserCreationForm

        # pai
        # fieldsets = (("User", {"fields": ("name",)}),) + auth_admin.UserAdmin.fieldsets
        fieldsets = auth_admin.UserAdmin.fieldsets
        list_display = _list_display
        search_fields = _search_fields

        # pai
        inlines = [ProfileInline]

else:
    @admin.register(User)
    class UserAdmin(auth_admin.UserAdmin):
        form = UserChangeForm
        add_form = UserCreationForm

        # pai
        # fieldsets = (("User", {"fields": ("name",)}),) + auth_admin.UserAdmin.fieldsets
        fieldsets = auth_admin.UserAdmin.fieldsets
        list_display = ["username", "email", "is_superuser"]
        search_fields = ["username", "email"]




from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model


User = get_user_model()

class Command(BaseCommand):
    help = 'Sets apigw_consumer_id to None'

    def handle(self, *args, **options):
        users = User.objects.all()
        users_count = users.count()
        updated_users = 0
        for user in users:
            user.apigw_consumer_id = None
            user.save()

        self.stdout.write(self.style.SUCCESS('Updated {0}/{1} Users'.format(users_count, updated_users)))



import logging

from rest_framework import permissions


def is_staff(user):
    return user and user.is_staff


class OwnerPermission(permissions.IsAuthenticated):

    message = 'You must be the owner.'

    def has_object_permission(self, request, view, obj):
        if (getattr(request, 'user', None) is not None and request.user == obj):
            return True
        return False


class StaffPermission(permissions.IsAuthenticated):
    message = 'You must be a staff member.'

    def has_permission(self, request, view):
        if is_staff(request.user):
            return True
        return False


class OwnerOrStaffPermission(permissions.IsAuthenticated):

    message = 'You must be the owner or a staff member.'

    def has_object_permission(self, request, view, obj):
        if request.user and ((request.user == obj) or is_staff(request.user)):
            return True
        return False


class PublicObjectOrOwnerOrStaffPermission(permissions.IsAuthenticated):

    def has_object_permission(self, request, view, obj):
        user = request.user
        user_is_staff = is_staff(user)

        if user_is_staff:
            return True

        is_public = getattr(obj, 'is_public', False)
        has_owner = getattr(obj, 'user', False)

        if (is_public and not has_owner) and not user_is_staff:
            return False

        if is_public:
            return True

        obj_user = getattr(obj, 'user', None)

        return obj_user == user



from django.apps import AppConfig
from django.conf import settings

if settings.DJANGO_SSO_BACKEND_ENABLED:
    from django_sso_app.core.apps.profiles.apps import ProfilesConfig as DjangoSSoAppProfilesConfig

    class ProfilesConfig(DjangoSSoAppProfilesConfig):
        name = 'django_sso_app.backend.profiles'
else:
    class ProfilesConfig(AppConfig):
        name = 'backend.profiles'



from django.conf.urls import url

from ...views.rest_auth import *


rest_auth_urlpatterns = [
    # rest_auth
    url(r'^api/v1/auth/registration/$', RegisterAPIView.as_view(), name='rest_register'),
    url(r'^api/v1/auth/verify-email/$', VerifyEmailAPIView.as_view(), name='rest_verify_email'),
    # completely overrided
    # url(r'^api/v1/auth/registration/', include('rest_auth.registration.urls')),

    # rest_auth
    url(r'^api/v1/auth/login/', LoginAPIView.as_view(), name='rest_login'),
    url(r'^api/v1/auth/logout/', LogoutAPIView.as_view(), name='rest_logout'),

    url(r'^api/v1/auth/user/', UserDetailsAPIView.as_view(), name='user_helper_view'),

    url(r'^api/v1/auth/password/reset/$', PasswordResetAPIView.as_view(), name='rest_password_reset'),
    url(r'^api/v1/auth/password/reset/confirm/$', PasswordResetConfirmAPIView.as_view(),
        name='rest_password_reset_confirm'),
    url(r'^api/v1/auth/password/change/$', PasswordChangeAPIView.as_view(),
        name='rest_password_change'),
    # completely overrided
    # url(r'^api/v1/auth/', include('rest_auth.urls')),
]



# Generated by Django 2.2.4 on 2019-09-13 18:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('fingerprint', models.CharField(max_length=32)),
                ('user_agent', models.CharField(blank=True, max_length=32, null=True)),
                ('apigw_jwt_id', models.CharField(blank=True, max_length=36, null=True)),
                ('apigw_jwt_key', models.CharField(blank=True, max_length=32, null=True)),
                ('apigw_jwt_secret', models.CharField(blank=True, max_length=32, null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='devices', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging

from rest_framework import serializers

from .models import Service, Subscription

logger = logging.getLogger('services')


class ServiceSerializer(serializers.ModelSerializer):
    is_subscribed = serializers.SerializerMethodField(required=False)
    is_unsubscribed = serializers.SerializerMethodField(required=False)
    tos = serializers.SerializerMethodField(required=False)

    class Meta:
        model = Service
        read_only_fields = ('id', 'is_subscribed', 'picture', 'is_unsubscribed')
        fields = read_only_fields + ('url', 'name',
                                     'role', 'cookie_domain',
                                     'redirect_wait',
                                     'tos',
                                     'subscription_required')

    def get_is_subscribed(self, instance):
        request = self.context.get("request")
        user = request.user
        if user.is_anonymous:
            return False
        return request.user.subscriptions.filter(service=instance).count() > 0

    def get_is_unsubscribed(self, instance):
        request = self.context.get("request")
        user = request.user
        if user.is_anonymous:
            return False
        subscription = request.user.subscriptions.filter(service=instance).first()
        if subscription is not None:
            return subscription.unsubscribed_at is not None
        return False

    def get_tos(self, instance):
        request = self.context.get('request')
        language = request.LANGUAGE_CODE
        tos = instance.get_tos(language)

        if tos is None:
            instance.get_tos('en')

        if tos is None:
            return None
        return tos.text


class SubscriptionSerializer(serializers.ModelSerializer):
    service = ServiceSerializer()

    class Meta:
        model = Subscription
        fields = ('id', 'service', 'created_at', 'is_unsubscribed')




import pytest

from django_sso_app.backend.users.forms import UserCreationForm
from django_sso_app.backend.users.tests.factories import UserFactory

pytestmark = pytest.mark.django_db


class TestUserCreationForm:
    def test_clean_username(self):
        # A user with proto_user params does not exist yet.
        proto_user = UserFactory.build()

        form = UserCreationForm(
            {
                "username": proto_user.username,
                "password1": proto_user._password,
                "password2": proto_user._password,
            }
        )

        assert form.is_valid()
        assert form.clean_username() == proto_user.username

        # Creating a user.
        form.save()

        # The user with proto_user params already exists,
        # hence cannot be created.
        form = UserCreationForm(
            {
                "username": proto_user.username,
                "password1": proto_user._password,
                "password2": proto_user._password,
            }
        )

        assert not form.is_valid()
        assert len(form.errors) == 1
        assert "username" in form.errors



import logging

from django.contrib.auth.models import Group as GroupModel
from django.urls import reverse

logger = logging.getLogger('groups')


class Group(GroupModel):
    class Meta:
        proxy = True

    def get_absolute_url(self):
        return reverse("group:detail", args=[self.pk])



import logging
import os

from allauth.account import views as allauth_account_views
from django.conf import settings
from django.views.generic import TemplateView

logger = logging.getLogger('backend')

CURRENT_DIR = os.getcwd()


# allauth views

class LoginView(allauth_account_views.LoginView):
    def get(self, *args, **kwargs):
        logger.info('Getting login')
        try:
            return super(LoginView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting login')
            raise

    def post(self, *args, **kwargs):
        logger.info('Logging in')
        try:
            return super(LoginView, self).post(*args, **kwargs)
        except:
            logger.exception('Error logging in')
            raise


class SignupView(allauth_account_views.SignupView):
    def get(self, *args, **kwargs):
        logger.info('Getting signup')
        try:
            return super(SignupView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting signup')
            raise

    def post(self, *args, **kwargs):
        logger.info('Signing up')
        try:
            return super(SignupView, self).post(*args, **kwargs)
        except:
            logger.exception('Error signin up')
            raise


class ConfirmEmailView(allauth_account_views.ConfirmEmailView):
    def get(self, *args, **kwargs):
        logger.info('Getting confirm email')
        try:
            return super(ConfirmEmailView, self).get(*args, **kwargs)
        except:
            logger.exception('Error get confirm email')
            raise

    def post(self, *args, **kwargs):
        logger.info('Confirming email')
        try:
            return super(ConfirmEmailView, self).post(*args, **kwargs)
        except:
            logger.exception('Error confirming email')
            raise


class EmailView(allauth_account_views.EmailView):
    def get(self, *args, **kwargs):
        logger.info('Getting email')
        try:
            return super(EmailView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting email')
            raise

    def post(self, *args, **kwargs):
        logger.info('Email')
        try:
            return super(EmailView, self).post(*args, **kwargs)
        except:
            logger.exception('Error email')
            raise


class PasswordChangeView(allauth_account_views.PasswordChangeView):
    def get(self, *args, **kwargs):
        logger.info('Getting password change')
        try:
            return super(PasswordChangeView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting password change')
            raise

    def post(self, *args, **kwargs):
        logger.info('Changing password')
        try:
            return super(PasswordChangeView, self).post(*args, **kwargs)
        except:
            logger.exception('Error changing password')
            raise


class PasswordSetView(allauth_account_views.PasswordSetView):
    def get(self, *args, **kwargs):
        logger.info('Getting password set')
        try:
            return super(PasswordSetView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting password set')
            raise

    def post(self, *args, **kwargs):
        logger.info('Setting password')
        try:
            return super(PasswordSetView, self).post(*args, **kwargs)
        except:
            logger.exception('Error setting password')
            raise


class PasswordResetView(allauth_account_views.PasswordResetView):
    def get(self, *args, **kwargs):
        logger.info('Getting ask password reset')
        try:
            return super(PasswordResetView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting ask password reset')
            raise

    def post(self, *args, **kwargs):
        logger.info('Asking for password reset')
        try:
            return super(PasswordResetView, self).post(*args, **kwargs)
        except:
            logger.exception('Error while asking password reset')
            raise


class PasswordResetDoneView(allauth_account_views.PasswordResetDoneView):
    def get(self, *args, **kwargs):
        logger.info('Getting password reset done')
        try:
            return super(PasswordResetDoneView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting password reset done')
            raise


class PasswordResetFromKeyView(allauth_account_views.PasswordResetFromKeyView):
    def get(self, *args, **kwargs):
        logger.info('Getting password reset from key')
        try:
            return super(PasswordResetFromKeyView, self).get(*args, **kwargs)
        except:
            logger.exception('Error while getting password reset from key')
            raise

    def post(self, *args, **kwargs):
        logger.info('Resetting password form key')
        try:
            return super(PasswordResetFromKeyView, self).post(*args, **kwargs)
        except:
            logger.exception('Error while resetting password form key')
            raise


class PasswordResetFromKeyDoneView(
    allauth_account_views.PasswordResetFromKeyDoneView):
    def get(self, *args, **kwargs):
        logger.info('Getting password reset from key done')
        try:
            return super(PasswordResetFromKeyDoneView, self).get(*args,
                                                                 **kwargs)
        except:
            logger.exception('Error while getting password reset from key done')
            raise


class LogoutView(allauth_account_views.LogoutView):
    def get(self, *args, **kwargs):
        logger.info('Getting logout')
        try:
            return super(LogoutView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting logout')
            raise

    def post(self, *args, **kwargs):
        logger.info('Logging out')
        try:
            return super(LogoutView, self).post(*args, **kwargs)
        except:
            logger.exception('Error logging out')
            raise


class AccountInactiveView(allauth_account_views.AccountInactiveView):
    def get(self, *args, **kwargs):
        logger.info('Getting account inactive')
        try:
            return super(AccountInactiveView, self).get(*args, **kwargs)
        except:
            logger.exception('Error while getting account inactive')
            raise


class EmailVerificationSentView(
    allauth_account_views.EmailVerificationSentView):
    def get(self, *args, **kwargs):
        logger.info('Getting email verification sent')
        try:
            return super(EmailVerificationSentView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting verification sent')
            raise


# custom views

class WebpackBuiltTemplateView(TemplateView):
    """
    Base built template view
    """
    template_name = "frontend/built.html"

    def get_context_data(self, *args, **kwargs):
        context = super(WebpackBuiltTemplateView, self).get_context_data(*args,
                                                                         **kwargs)
        context['COOKIE_DOMAIN'] = settings.COOKIE_DOMAIN
        context['DJANGO_SSO_DEFAULT_REFERRER'] = settings.DJANGO_SSO_DEFAULT_REFERRER
        return context


class ProfileView(TemplateView):
    template_name = "pages/profile.html"

    def get(self, *args, **kwargs):
        logger.info('Getting profile')
        try:
            return super(ProfileView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting profile')
            raise


class ProfileUpdateView(TemplateView):

    def get(self, *args, **kwargs):
        logger.info('Getting profile update')
        try:
            return super(ProfileUpdateView, self).get(*args, **kwargs)
        except:
            logger.exception('Error getting profile update')
            raise



# Generated by Django 2.2.4 on 2019-09-13 18:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('name', models.CharField(max_length=32, null=True, unique=True)),
                ('url', models.CharField(max_length=255, unique=True)),
                ('picture', models.TextField(blank=True, null=True, verbose_name='picture')),
                ('cookie_domain', models.CharField(blank=True, max_length=255, null=True)),
                ('redirect_wait', models.PositiveIntegerField(default=2000)),
                ('subscription_required', models.BooleanField(default=False)),
                ('is_public', models.BooleanField(default=True)),
                ('role', models.SmallIntegerField(blank=True, choices=[(-1, 'Staff'), (0, 'Cityzen'), (1, 'Institutional'), (2, 'Professionist'), (4, 'Company'), (6, 'Istitutional Partition Item'), (7, 'App')], null=True)),
            ],
        ),
        migrations.CreateModel(
            name='TOS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(max_length=3)),
                ('text', models.TextField()),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='terms_of_service', to='services.Service')),
            ],
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('unsubscribed_at', models.DateTimeField(blank=True, null=True)),
                ('service', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to='services.Service')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='subscriptions', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import logging
import random
import string
from importlib import import_module

from django.conf import settings
from django.contrib.auth import get_user_model
from six import string_types

logger = logging.getLogger("utils")


def import_callable(path_or_callable):
    if hasattr(path_or_callable, '__call__'):
        return path_or_callable
    else:
        assert isinstance(path_or_callable, string_types)
        package, attr = path_or_callable.rsplit('.', 1)
        return getattr(import_module(package), attr)


def get_profile_model():
    profile_model = getattr(settings, 'DJANGO_SSO_APP_USER_PROFILE_MODEL', None)
    if profile_model is None:
        return get_user_model()
    else:
        return import_callable(profile_model)


def get_country_language(country):
    logger.debug('Getting language for country {}'.format(country))
    if country == 'IT':
        return 'it'
    elif country == 'BR' or country == 'PT':
        return 'pt'
    elif country == 'ES':
        return 'es'

    return 'en-gb'


def get_random_string(N):
    return ''.join(random.choice(string.ascii_uppercase + string.digits) for _ in range(N))



import pytest
from django.conf import settings

pytestmark = pytest.mark.django_db

"""
def test_user_get_absolute_url(user: settings.AUTH_USER_MODEL):
    assert user.get_absolute_url() == f"/users/{user.username}/"
"""


from django.conf.urls import url
from django.contrib.auth.decorators import login_required

from ...views.allauth import *


allauth_urlpatterns = [
    url(r'^profile/update/$', ProfileUpdateView.as_view(), name='sso_profile_update'),

    url(r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        TemplateView.as_view(),
        name='restauth_password_reset_confirm'),

    # allauth
    url(r"^signup/$", SignupView.as_view(), name="account_signup"),
    url(r"^login/$", LoginView.as_view(), name="account_login"),
    url(r"^logout/$", LogoutView.as_view(), name="account_logout"),

    url(r"^password/change/$", login_required(PasswordChangeView.as_view()),
        name="account_change_password"),
    url(r"^password/set/$", login_required(PasswordSetView.as_view()), name="account_set_password"),

    url(r"^inactive/$", AccountInactiveView.as_view(), name="account_inactive"),

    # E-mail
    url(r"^email/$", login_required(EmailView.as_view()), name="account_email"),
    url(r"^confirm-email/$", EmailVerificationSentView.as_view(),
        name="account_email_verification_sent"),
    url(r"^confirm-email/(?P<key>[-:\w]+)/$", ConfirmEmailView.as_view(),
        name="account_confirm_email"),

    # password reset
    url(r"^password/reset/$", PasswordResetView.as_view(),
        name="account_reset_password"),
    url(r"^password/reset/done/$", PasswordResetDoneView.as_view(),
        name="account_reset_password_done"),
    url(r"^password/reset/key/(?P<uidb36>[0-9A-Za-z]+)/(?P<key>.+)/$",
        PasswordResetFromKeyView.as_view(),
        name="account_reset_password_from_key"),
    url(r"^password/reset/key/done/$", PasswordResetFromKeyDoneView.as_view(),
        name="account_reset_password_from_key_done"),
    # completely overrided
    # url(r'^$', include('allauth.urls')),
]



from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns

from .views import ServiceApiViewSet


_urlpatterns = [

    url(r'^$',
        ServiceApiViewSet.as_view({'get': 'list'}),
        name='services_list'),

    url(r'^(?P<pk>\d+)/$',
        ServiceApiViewSet.as_view({'get': 'retrieve'}),
        name='services_detail'),

    url(r'^(?P<pk>\d+)/subscribe/$',
        ServiceApiViewSet.as_view({'post': 'subscribe'}),
        name='services_subscription'),

]

urlpatterns = format_suffix_patterns(_urlpatterns)



# Generated by Django 2.2.4 on 2019-09-13 18:24

import django.contrib.auth.models
from django.db import migrations


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.group',),
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
    ]



import logging

import jwt
from allauth.account.signals import (email_added, password_reset)
from django.contrib.auth import get_user_model
from django.contrib.auth.signals import user_logged_in
from rest_framework_jwt.settings import api_settings
from rest_framework_jwt.utils import jwt_payload_handler as rest_framework_jwt_payload_handler

Users = get_user_model()
logger = logging.getLogger('backend')


def jwt_payload_handler(user, device=None):
    payload = rest_framework_jwt_payload_handler(user)
    if device is not None:
        # removing email from payload
        user_email = payload.pop('email', None)
        payload['user_id'] = str(user.sso_id)
        payload['rev'] = user.sso_rev
        payload['fp'] = device.fingerprint
        payload['iss'] = device.apigw_jwt_key

    return payload


def jwt_encode_handler(payload, device=None):
    if device is not None:
        key = device.apigw_jwt_secret
        return jwt.encode(
            payload,
            key,
            api_settings.JWT_ALGORITHM
        ).decode('utf-8')
    else:
        raise Exception('Encoding without device')
        # return rest_framework_jwt_encode_handler(payload)


def jwt_decode_handler(token, device=None):
    options = {
        'verify_exp': api_settings.JWT_VERIFY_EXPIRATION,
    }

    if device is None:
        raise Exception('Decoding without device')

        # get user from token, BEFORE verification, to get user secret key

        unverified_payload = jwt.decode(token, None, False)
        secret_key = jwt_get_secret_key(unverified_payload)

        return jwt.decode(
            token,
            api_settings.JWT_PUBLIC_KEY or secret_key,
            api_settings.JWT_VERIFY,
            options=options,
            leeway=api_settings.JWT_LEEWAY,
            audience=api_settings.JWT_AUDIENCE,
            issuer=api_settings.JWT_ISSUER,
            algorithms=[api_settings.JWT_ALGORITHM]
        )


def jwt_encode(user, device=None):
    payload = jwt_payload_handler(user, device)

    return jwt_encode_handler(payload, device)


def login_handler(sender, user, request, **kwargs):
    if hasattr(request, 'data'):
        fingerprint = request.data.get('fingerprint')
        logger.info('User {0} logged in with fingerprint {1}'.format(user, fingerprint))


def email_added_handler(sender, request, email_address, **kwargs):
    logger.info('User {0} created unconfirmed email {1}'.format(request.user, email_address))


def password_reset_handler(sender, request, user, **kwargs):
    logger.info('User {0} resetted email'.format(user))


user_logged_in.connect(login_handler)
email_added.connect(email_added_handler)
password_reset.connect(password_reset_handler)



from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model

from allauth.account.adapter import get_adapter

User = get_user_model()


class Command(BaseCommand):
    help = 'Deletes all devices (and jwts)'

    def handle(self, *args, **options):
        users = User.objects.all()
        users_count = users.count()
        updated_users = 0
        adapter = get_adapter()
        for user in users:
            devices_count = user.devices.count()
            deleted_devices_count = adapter.remove_all_user_devices(user)

            self.stdout.write(self.style.SUCCESS('Deleted {0}/{1} user devices for "{2}"'.format(devices_count, deleted_devices_count, user)))
            
            updated_users += 1

        self.stdout.write(self.style.SUCCESS('Updated {0}/{1} Users'.format(users_count, updated_users)))



# Generated by Django 2.2.4 on 2019-09-13 18:24

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={},
        ),
        migrations.AddField(
            model_name='user',
            name='apigw_consumer_id',
            field=models.CharField(blank=True, max_length=36, null=True, verbose_name='apigw consumer id'),
        ),
        migrations.AddField(
            model_name='user',
            name='sso_id',
            field=models.UUIDField(db_index=True, default=uuid.uuid4, editable=False, unique=True, verbose_name='sso id'),
        ),
        migrations.AddField(
            model_name='user',
            name='sso_rev',
            field=models.PositiveIntegerField(default=1, verbose_name='sso revision'),
        ),
        migrations.AddField(
            model_name='user',
            name='unsubscribed_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='unsubscription date'),
        ),
    ]



import pytest
from celery.result import EagerResult


from django_sso_app.backend.users.tasks import get_users_count
from django_sso_app.backend.users.tests.factories import UserFactory


@pytest.mark.django_db
def test_user_count(settings):
    """A basic test to execute the get_users_count Celery task."""
    UserFactory.create_batch(3)
    settings.CELERY_TASK_ALWAYS_EAGER = True
    task_result = get_users_count.delay()
    assert isinstance(task_result, EagerResult)
    assert task_result.result == 3



from django.conf import settings
# from meta.views import Meta


def emails_domain_settings(request):
    return {
        "EMAILS_DOMAIN": settings.EMAILS_DOMAIN,
        "EMAILS_SITE_NAME": settings.EMAILS_SITE_NAME
    }

def google_api_settings(request):
    return {
        "GOOGLE_API_KEY": settings.GOOGLE_API_KEY,
        "GOOGLE_MAPS_API_VERSION": settings.GOOGLE_MAPS_API_VERSION,
    }

def google_analytics_settings(request):
    return {
        "GOOGLE_ANALYTICS_TRACKING_ID": settings.GOOGLE_ANALYTICS_TRACKING_ID,
    }

def raven_js_dsn_settings(request):
    return {
        "RAVEN_JS_DSN": settings.RAVEN_JS_DSN,
    }

def disable_js(request):
    is_active = request.GET.get('nojs', None)
    return {
        "DISABLE_JS": (is_active is not None) or settings.DJANGO_SSO_NO_CUSTOM_FRONTEND_APP,
    }

def get_repository_rev(request):
    return {
        "REPOSITORY_REV": settings.REPOSITORY_REV
    }

"""
def django_meta(request):
    return {
        'meta': Meta(
           description=settings.META_DEFAULT_DESCRIPTION,
        )
    }
"""

def sso_cookie_domain(request):
    return {
        "sso_cookie_domain": settings.DJANGO_SSO_COOKIE_DOMAIN
    }

def get_auth_settings(request):
    return {
        "DJANGO_ALLAUTH_ENABLED": settings.DJANGO_ALLAUTH_ENABLED
    }



import logging

from allauth.account.adapter import get_adapter
from django.conf import settings
from django.contrib.auth import get_user_model
from django.http import Http404
from django.shortcuts import get_object_or_404
from rest_framework import mixins
from rest_framework import permissions
from rest_framework import status
from rest_framework import viewsets
from rest_framework.response import Response

from django_sso_app.backend.utils import get_request_jwt, update_response_jwt
from .models import Service
from .serializers import SubscriptionSerializer, ServiceSerializer
from ..users.serializers import UserUnsubscriptionSerializer, UserSerializer
from ..users.views import TryAuthenticateMixin

logger = logging.getLogger('services')
User = get_user_model()


class ServiceApiViewSet(mixins.ListModelMixin,
                        mixins.RetrieveModelMixin,
                        viewsets.GenericViewSet):
    """
    Service API viewset
    """

    serializer_class = ServiceSerializer
    permission_classes = (permissions.AllowAny, )

    def get_queryset(self):
        return Service.objects.filter(is_public=True)

    def subscribe(self, request, *args, **kwargs):
        pk = self.kwargs.get('pk')
        user = request.user
        service = Service.objects.get(id=pk)
        adapter = get_adapter(request)

        if service is None:
            raise Http404

        if user.subscriptions.filter(service=service).count() > 0:
            return Response('Already subscribed.', status=status.HTTP_409_CONFLICT)

        subscription_created = adapter.subscribe_to_service(user, service.url, update_rev=True)

        if subscription_created:
            response = Response('Subscribed.', status=status.HTTP_201_CREATED)
        else:
            return Response('Subscription error.', status=status.HTTP_500_SERVER_ERROR)

        if not settings.DEBUG:
            try:
                received_jwt = get_request_jwt(request)
                update_response_jwt(received_jwt, user, request, response)
            except KeyError as e:
                return Response(str(e), status=status.HTTP_400_BAD_REQUEST)

        return response


class SubscriptionApiViewSet(mixins.RetrieveModelMixin,
                            mixins.ListModelMixin,
                            TryAuthenticateMixin,
                            viewsets.GenericViewSet):

    serializer_class = SubscriptionSerializer

    def get_queryset(self):
        queryset = self.request.user.subscriptions.all()
        return queryset

    def unsubscribe(self, request, *args, **kwargs):
        user_pk = self.kwargs.get('user_pk')
        pk = self.kwargs.get('pk')
        
        user = get_object_or_404(User.objects.all(), sso_id=user_pk)
        
        if user != request.user:
            return Response('Caller must be same user.', status=status.HTTP_400_BAD_REQUEST)
        
        serializer = UserUnsubscriptionSerializer(data=request.data, context={'request': request})

        if not serializer.is_valid():
            return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

        username = user.username
        email = user.email
        password = serializer.data.get('password')

        user = self.try_authenticate(username, email, password)

        subscription = self.get_queryset().filter(id=pk).first()
        logger.info('User {0} wants to unregister from service {1}'.format(user, subscription))

        adapter = get_adapter()
        adapter.unsubscribe_from_service(user, subscription, True) # user devices will be removed by User pre_save signal

        serializer = UserSerializer(user, context={'request': request})
        response = Response(serializer.data)


        if not settings.DEBUG:
          try:
              received_jwt = get_request_jwt(request)
              update_response_jwt(received_jwt, user, request, response)
          except KeyError as e:
              return Response(str(e), status=status.HTTP_400_BAD_REQUEST)

        return response

    def create(self, request, *args, **kwargs):
        user_pk = self.kwargs.get('user_pk')
        service_pk = self.kwargs.get('service_pk')

        requesting_user = request.user
        user = User.objects.get(sso_id=user_pk)
        requested_from_different_user = requesting_user != user
        
        if requested_from_different_user and not requesting_user.is_staff:
            return Response('Unauthorized.', status=status.HTTP_401_UNAUTHORIZED)

        service = Service.objects.get(id=service_pk)
        if service is None:
            raise Http404

        if user.subscriptions.filter(service=service).count() > 0:
            return Response('Already subscribed.', status=status.HTTP_409_CONFLICT)

        adapter = get_adapter(request)
        subscription_created = adapter.subscribe_to_service(user, service.url, update_rev=True)

        if subscription_created:
            response = Response('Subscribed.', status=status.HTTP_201_CREATED)
        else:
            return Response('Subscription error.', status=status.HTTP_500_SERVER_ERROR)

        if not requested_from_different_user:
            if not settings.DEBUG:
                try:
                    received_jwt = get_request_jwt(request)
                    update_response_jwt(received_jwt, user, request, response)
                except KeyError as e:
                    return Response(str(e), status=status.HTTP_400_BAD_REQUEST)
        
        return response




from django.apps import AppConfig


class GroupsConfig(AppConfig):
    name = 'django_sso_app.core.apps.groups'

    def ready(self):
        try:
            import django_sso_app.core.apps.groups.signals  # noqa F401
        except ImportError:
            pass



import logging

from django.contrib.auth import get_user_model
from django.db import models
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from django_sso_app.core.apps.profiles.models import AbstractUserProfileModel

logger = logging.getLogger('profiles')
User = get_user_model()


class Profile(AbstractUserProfileModel):
    user = models.OneToOneField(User, on_delete=models.CASCADE,
                                verbose_name=_("user"),
                                related_name="sso_app_profile")

    def get_absolute_url(self):
        return reverse("profiles:detail", args=[self.sso_id])



import string

from django.conf import settings

all_chars = string.ascii_letters + string.digits
forbidden_chars = ('I', 'l', '1', '|', '0', 'O')

permitted_chars = [el for el in all_chars if el not in forbidden_chars]

CODE_LENGTH = getattr(settings, 'COUPONS_CODE_LENGTH', 36)
CODE_CHARS = getattr(settings, 'COUPONS_CODE_CHARS', permitted_chars)



from django.core.management.base import BaseCommand, CommandError
from django.contrib.auth import get_user_model
from allauth.account.adapter import get_adapter


User = get_user_model()

class Command(BaseCommand):
    help = 'Deletes all user devices (and jwts)'

    def add_arguments(self, parser):
        parser.add_argument('username', nargs='+', type=str)

    def handle(self, *args, **options):
        for username in options['username']:
            try:
                user = User.objects.get(username=username)
            except User.DoesNotExist:
                raise CommandError('User "%s" does not exist' % username)

            adapter = get_adapter()
            devices_count = user.devices.count()
            deleted_devices_count = adapter.remove_all_user_devices(user)

            self.stdout.write(self.style.SUCCESS('Deleted {0}/{1} user devices for "{2}"'.format(devices_count, deleted_devices_count, username)))



from rest_framework import viewsets
from rest_framework import permissions

from .models import Group
from .serializers import GroupSerializer


class GroupViewSet(viewsets.ReadOnlyModelViewSet):
    serializer_class = GroupSerializer
    queryset = Group.objects.all()
    lookup_field = 'pk'

    def list(self, request, *args, **kwargs):
        """
        List groups
        """
        return super(GroupViewSet, self).list(request, *args, **kwargs)

    def retrieve(self, request, pk=None, *args, **kwargs):
        """
        Group detail
        """
        return super(GroupViewSet, self).retrieve(request, pk, *args, **kwargs)



import logging

from django.conf import settings
from django.contrib import auth
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.utils.deprecation import \
    MiddlewareMixin  # https://stackoverflow.com/questions/42232606/django
# -exception-middleware-typeerror-object-takes-no-parameters
from django.utils.encoding import smart_str
from django.utils.http import urlencode
from jwt import decode
from jwt.exceptions import DecodeError

from django_sso_app.core.utils import get_profile_model
from .utils import (get_sso_id,
                    get_profile_from_sso,
                    update_local_profile_from_sso)

logger = logging.getLogger("app")
Profile = get_profile_model()


class DjangoSsoAppMiddleware(MiddlewareMixin):
    """
    See django.contrib.auth.middleware.RemoteUserMiddleware.
    """

    # Name of request header to grab username from.  This will be the key as
    # used in the request.META dictionary, i.e. the normalization of headers to
    # all uppercase and the addition of "HTTP_" prefix apply.
    header = settings.DJANGO_SSO_CONSUMER_USERNAME_HEADER
    # Username for anonymous user
    anonymous_username = settings.DJANGO_SSO_ANONYMOUS_CONSUMER_USERNAME

    def process_request(self, request):
        # AuthenticationMiddleware is required so that request.user exists.

        if not hasattr(request, "user"):
            raise ImproperlyConfigured(
                "SSO middleware requires the authentication middleware to be"
                " installed.  Edit your MIDDLEWARE setting to insert"
                " 'django.contrib.auth.middleware.AuthenticationMiddleware'"
                " before the SsoMiddleware class.")

        if request.user.is_authenticated and request.user.is_staff:  # django
            #  >= 1.10 has is_authenticated as parameter
            # If a staff user is already authenticated, we don't need to
            # continue
            return

        if settings.DJANGO_SSO_APP_ENABLED:
            logger.info('DJANGO_SSO_APP_ENABLED')
        if settings.DJANGO_SSO_BACKEND_ENABLED:
            logger.info('DJANGO_SSO_BACKEND_ENABLED')

        try:
            consumer_username = request.META[self.header]

            if consumer_username == self.anonymous_username:
                if request.user.is_authenticated:
                    self._remove_invalid_user(request)
                return

            encoded_jwt = request.COOKIES.get(settings.DJANGO_SSO_JWT_HEADER,
                                              None)

        except KeyError as e:
            # If specified header or jwt doesn't exist then remove any existing
            # authenticated user, or return (leaving request.user set to
            # AnonymousUser by the AuthenticationMiddleware).
            if request.user.is_authenticated:
                self._remove_invalid_user(request)
            return

        if settings.DJANGO_SSO_REPLICATE_PROFILE:
            try:
                decoded_jwt = decode(jwt=encoded_jwt, verify=False)
            except DecodeError:
                logger.exception("Error decoding JWT!")

                if request.user.is_authenticated():
                    self._remove_invalid_user(request)
                return
        else:
            decoded_jwt = None

        # If the user is already authenticated and that user is active and the
        # one we are getting passed in the headers, then the correct user is
        # already persisted in the session and we don't need to continue.
        if request.user.is_authenticated:
            sso_id = get_sso_id(consumer_username)
            user_profile = request.user

            if settings.DJANGO_SSO_APP_USES_EXTERNAL_USER_PROFILE:
                user_profile = Profile.objects.get(user=request.user)

            if user_profile.sso_id == sso_id:
                if request.user.is_active:
                    if settings.DJANGO_SSO_REPLICATE_PROFILE:
                        # Check if sso_rev changed and, if so, fetch profile
                        # from
                        # SSO and update the local one
                        rev_changed = user_profile.sso_rev != decoded_jwt["rev"]
                        if rev_changed:
                            logger.info("Rev changed from {} to {} for {},"
                                        " updating ..."
                                        .format(user_profile.sso_rev,
                                                decoded_jwt["rev"],
                                                consumer_username))

                            sso_profile = get_profile_from_sso(
                                sso_id=sso_id, encoded_jwt=encoded_jwt)
                            update_local_profile_from_sso(
                                sso_profile=sso_profile,
                                profile=user_profile)

                            logger.info("{} updated with latest data from SSO"
                                        .format(consumer_username))

                        return
                    else:
                        logger.info("{} logged in"
                                    .format(consumer_username))
                        return

                # The user is not active.
                self._remove_invalid_user(request)
            else:
                # An authenticated user is associated with the request, but
                # it does not match the authorized user in the header.
                self._remove_invalid_user(request)

        # We are seeing this user for the first time in this session, attempt
        # to authenticate the user.
        user = auth.authenticate(request=request,
                                 consumer_username=consumer_username,
                                 encoded_jwt=encoded_jwt,
                                 decoded_jwt=decoded_jwt)

        # Redirect user to Term Of Service.
        is_to_subscribe = getattr(user, "is_to_subscribe", False)
        _username = smart_str(user.username)
        logger.info('IS {} TO SUBSCRIBE? {}'
                    .format(_username, is_to_subscribe))
        if user and is_to_subscribe:
            _qs = urlencode({"next": settings.DJANGO_SSO_SERVICE_URL})
            _url = "{sso}?{qs}".format(sso=settings.DJANGO_SSO_LOGIN_URL,
                                       qs=_qs)
            logger.info("User {username} must agree to the Terms of Service,"
                        " redirecting to {url} ..."
                        .format(username=_username, url=_url))
            response = HttpResponseRedirect(redirect_to=_url)
            return response

        if user and user.is_active:
            # User is valid.  Set request.user and persist user in the session
            # by logging the user in.
            request.user = user
            auth.login(request, user)
            logger.info("User {username} was authenticated successfully!"
                        .format(username=_username))

    @staticmethod
    def _remove_invalid_user(request):
        """
        Removes the current authenticated user in the request which is invalid.
        """
        auth.logout(request)



# Generated by Django 2.2.4 on 2019-09-13 18:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, verbose_name='is active')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='updated at')),
                ('sso_id', models.CharField(blank=True, db_index=True, max_length=36, null=True, unique=True)),
                ('sso_rev', models.PositiveIntegerField(default=0)),
                ('first_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=30, null=True, verbose_name='last name')),
                ('role', models.SmallIntegerField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('picture', models.TextField(blank=True, null=True, verbose_name='picture')),
                ('birthdate', models.DateField(blank=True, null=True, verbose_name='birthdate')),
                ('latitude', models.FloatField(blank=True, null=True, verbose_name='latitude')),
                ('longitude', models.FloatField(blank=True, null=True, verbose_name='longitude')),
                ('country', models.CharField(blank=True, max_length=46, null=True, verbose_name='country')),
                ('address', models.TextField(blank=True, null=True, verbose_name='address')),
                ('unsubscribed_at', models.DateTimeField(blank=True, null=True)),
                ('language', models.CharField(blank=True, max_length=3, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='sso_app_profile', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging

from django.db.models.signals import post_save
from django.dispatch import receiver
from django.conf import settings

from django_sso_app.core.utils import get_profile_model

logger = logging.getLogger('profiles')
Profile = get_profile_model()


@receiver(post_save, sender=Profile)
def profile_updated(sender, instance, created, **kwargs):
    if settings.DJANGO_SSO_BACKEND_ENABLED:
        if not created:
            logger.info('Profile updated for {}, updating user revision {}.'.format(instance.user, instance.sso_rev))

            setattr(instance.user, '__profile_field_updated', True)
            instance.user.update_rev(True)



import logging

from allauth.account.adapter import get_adapter
from allauth.account.models import EmailAddress
from django.conf import settings
from django.contrib.auth import get_user_model
from rest_framework import serializers

from django_sso_app.core.permissions import is_staff
from django_sso_app.core.serializers import AbsoluteUrlSerializer
from django_sso_app.core.utils import get_profile_model

logger = logging.getLogger('users')
User = get_user_model()
Profile = get_profile_model()


class CheckUserExistenceSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ('sso_id',)
        read_only_fields = ('sso_id',)


class UserSerializer(AbsoluteUrlSerializer):
    subscriptions = serializers.SerializerMethodField(required=False)
    email_verified = serializers.SerializerMethodField(required=False)

    groups = serializers.SerializerMethodField(required=False)

    profile = serializers.SerializerMethodField(required=False)

    class Meta:
        model = User
        read_only_fields = (
            'url',
            'sso_id', 'sso_rev', 'date_joined', 'username', 'is_unsubscribed',
            'is_active', 'subscriptions', 'email_verified',
            'groups',
            'profile')
        fields = read_only_fields

    def get_subscriptions(self, instance):
        serialized = []
        for el in instance.subscriptions.all():
            serialized.append({
                'url': el.service.url,
                'is_unsubscribed': el.is_unsubscribed
            })
        return serialized

    def get_email_verified(self, instance):
        if is_staff(instance):
            return True
        try:
            email_address = instance.emailaddress_set.get(email=instance.email)
            return email_address.verified
        except:
            logger.info('{} has not verified email.'.format(instance))
            pass
        finally:
            return False

    def get_groups(self, instance):
        groups = list(instance.groups.values_list('name', flat=True))

        return groups

    def get_profile(self, instance):
        profile = instance.sso_app_profile
        request = self.context['request']
        return request.build_absolute_uri(profile.get_absolute_url())

    def update(self, instance, validated_data):
        user = instance
        request = self.context.get("request")
        requesting_user = request.user
        requesting_user_is_staff = is_staff(requesting_user)
        skip_confirmation = request.GET.get('skip_confirmation', None)
        must_confirm_email = True

        logger.info(
            'User {0} wants to update user {1}: {2}'.format(requesting_user, user, validated_data))

        if requesting_user_is_staff and skip_confirmation is not None:
            must_confirm_email = False

        new_email = validated_data.get('email', None)
        if (new_email is not None):

            if new_email == user.email:  # !! because of app shitty logic (
                # always sends all fields)
                logger.info('{0} is has same email'.format(user))

            else:
                logger.info(
                    '{0} is changing email for user {1}, email confirmation '
                    'is {2}'.format(
                        requesting_user, user, must_confirm_email))

                adapter = get_adapter(request=request)

                created_email = EmailAddress.objects.add_email(request, user,
                                                               new_email,
                                                               confirm=must_confirm_email)
                logger.info(
                    'A new EmailAddress for {0} has been created with id {1}'
                    ''.format(
                        new_email, created_email.id))

                if must_confirm_email:
                    if requesting_user_is_staff:
                        logger.info(
                            'deactivating all emails for {0} while email has been updated by staff user {1}'.format(user,
                                                                                                               requesting_user))

                        # EmailAddress.objects.filter(user=user, primary=True).update(primary=False)
                        # setattr(request.session, 'account_verified_email', None)
                        setattr(user, '__email_updated', new_email)

                    logger.info(
                        '{0} must confirm email {1}'.format(user, new_email))
                else:
                    adapter.confirm_email(None, created_email)

                setattr(user, '__email_updated', True)

        new_password = validated_data.get('password', None)
        if (new_password is not None):
            logger.info(
                '{0} is changing password for user {1}'.format(requesting_user,
                                                               user))
            validated_data.pop(
                'password')  # serializer must NOT save the received plain
            # password, password update logic performed by user.set_password

            user.set_password(new_password)

        # devices will be removed by signal
        # actual_devices = user.devices.all()
        # actual_devices_count = actual_devices.count()
        # deleted_devices = user.remove_all_user_devices()

        return super(UserSerializer, self).update(instance, validated_data)

    def to_representation(self, obj):
        # get the original representation
        ret = super(UserSerializer, self).to_representation(obj)
        req = self.context.get('request', None)

        if req is not None:
            # remove field if password if not asked
            with_pass = req.query_params.get('with_password',
                                             not settings.DJANGO_SSO_HIDE_PASSWORD_FROM_USER_SERIALIZER)

            if not with_pass:
                ret.pop('password', None)

        # return the modified representation
        return ret


class UserUpdateSerializer(UserSerializer):
    class Meta:
        model = User
        read_only_fields = (
            'url',
            'sso_id', 'sso_rev', 'date_joined', 'username', 'is_unsubscribed',
            'is_active', 'subscriptions', 'email_verified',
            'groups',
            'profile')
        fields = read_only_fields + ('username', 'email', 'password')

    def update(self, instance, validated_data):
        user = instance
        request = self.context.get("request")
        requesting_user = request.user
        requesting_user_is_staff = is_staff(requesting_user)
        skip_confirmation = request.GET.get('skip_confirmation', None)
        must_confirm_email = True

        logger.info(
            'User {0} wants to update user {1}: {2}'.format(requesting_user, user, validated_data))

        if requesting_user_is_staff and skip_confirmation is not None:
            must_confirm_email = False

        new_email = validated_data.get('email', None)
        if (new_email is not None):

            if new_email == user.email:  # !! because of app shitty logic (
                # always sends all fields)
                logger.info('{0} is has same email'.format(user))

            else:
                logger.info(
                    '{0} is changing email for user {1}, email confirmation '
                    'is {2}'.format(
                        requesting_user, user, must_confirm_email))

                adapter = get_adapter(request)

                created_email = EmailAddress.objects.add_email(request, user,
                                                               new_email,
                                                               confirm=must_confirm_email)
                logger.info(
                    'A new EmailAddress for {0} has been created with id {1}'
                    ''.format(
                        new_email, created_email.id))

                if must_confirm_email:
                    if requesting_user_is_staff:
                        logger.info(
                            'deactivating all emails for {0} while email has been updated by staff user {1}'.format(user,
                                                                                                               requesting_user))

                        # EmailAddress.objects.filter(user=user, primary=True).update(primary=False)
                        # setattr(request.session, 'account_verified_email', None)
                        setattr(user, '__email_updated', new_email)

                    logger.info(
                        '{0} must confirm email {1}'.format(user, new_email))
                else:
                    adapter.confirm_email(None, created_email)

                setattr(user, '__email_updated', True)


        new_password = validated_data.get('password', None)
        if (new_password is not None):
            logger.info(
                '{0} is changing password for user {1}'.format(requesting_user,
                                                               user))

            password_is_hashed = request.query_params.get('password_is_hashed', None)
            if password_is_hashed:
                user.password = new_password
            else:
                user.set_password(new_password)
            validated_data.pop(
                'password')  # serializer must NOT save the received plain
            # password, password update logic performed by user.set_password

        # devices will be removed by signal
        # actual_devices = user.devices.all()
        # actual_devices_count = actual_devices.count()
        # deleted_devices = user.remove_all_user_devices()

        return super(UserSerializer, self).update(instance, validated_data)


class UserUnsubscriptionSerializer(serializers.Serializer):
    password = serializers.CharField()


class UserRevisionSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        read_only_fields = ('sso_id', 'sso_rev')
        fields = read_only_fields



from django.conf import settings
from django.test import TestCase
from allauth.account.models import EmailAddress
from django.contrib.auth import get_user_model

from ...devices.models import Device
from django_sso_app.core.utils import get_profile_model

User = get_user_model()
Profile = get_profile_model()


class UserTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create(username="pippo", email="pippo@disney.com")
        self.user_email = EmailAddress.objects.create(user=self.user,
                                                      email=self.user.email,
                                                      primary=True,
                                                      verified=True)
        self.user_device = Device.objects.create(user=self.user,
                                                 fingerprint="abc123",
                                                 apigw_jwt_id="1"
                                                 )

    def test_user_field_update_updates_rev(self):
        """
        User update updates user revision
        """
        user = self.user
        user_rev = user.sso_rev

        user.username = 'pipipipi'
        user.save()

        self.assertEqual(user.sso_rev, user_rev + 1)
